/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package customControls;

import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.fxml.FXML;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;
import javafx.util.Duration;

/**
 *
 * @author Martin
 */
public class KeyFieldsControl extends Pane {

    private KeyCell cells[][];

    private Text roundLabels[];

    public KeyFieldsControl() {
        GridPane.setHgrow(this, Priority.ALWAYS);
        GridPane.setVgrow(this, Priority.ALWAYS);
        VBox.setVgrow(this, Priority.ALWAYS);
        cells = new KeyCell[11][16];
        initialization();
        refresh(0);
    }

    private KeyFieldsControl getInstance() {
        return this;
    }

    private void initialization() {
        for (int i = 0; i < 11; i++) {
            for (int j = 0; j < 17; j++) {
                if (j == 0) {
                    LabelCell lc = new LabelCell(String.valueOf(i));
                    lc.layoutXProperty().bind(new SimpleDoubleProperty(0));
                    lc.layoutYProperty().bind(widthProperty().divide(14).multiply(i));
                    this.getChildren().add(lc);
                } else {
                    KeyCell kc = new KeyCell(String.format("%02X", i * 10 + j));

                    kc.layoutXProperty().bind(widthProperty().divide(18).multiply(j).add(widthProperty().divide(18).multiply((double) 1 / 3).multiply((j - 1) / 4)));
                    kc.layoutYProperty().bind(widthProperty().divide(14).multiply(i));
                    cells[i][j - 1] = kc;
                    getChildren().add(kc);
                }
            }
        }
    }

    public void setValues(byte[] key) {
        for (int i = 0; i < 11; i++) {
            for (int j = 0; j < 16; j++) {
                KeyCell kc = cells[i][j];
                kc.setText(String.format("%02X", key[i * 16 + j]));
            }
        }
    }

    public void refresh(int round) {
        for (int i = 0; i < 11; i++) {
            for (int j = 0; j < 16; j++) {
                KeyCell kc = cells[i][j];
                if (i == round) {
                    kc.rectangle.setFill(Color.rgb(253, 188, 66));
                } else {
                    kc.rectangle.setFill(Color.YELLOW);
                }
                kc.scaleXProperty().set(1.0);
                kc.scaleYProperty().set(1.0);
            }
        }
    }

    Timeline addHighlightingFrames(Timeline tl, int round, int cellOffset, int timeInMillis) {
        KeyCell sc = cells[round][cellOffset];
        sc.scaleXProperty().addListener(new ChangeListener<Number>() {

            @Override
            public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number newValue) {
                sc.toFront();
            }
        });

        KeyFrame kf99 = new KeyFrame(Duration.millis(timeInMillis),
                new KeyValue(sc.rectangle.fillProperty(), Color.rgb(253, 188, 66))
        );

        timeInMillis += 1000;
        KeyFrame kf100 = new KeyFrame(Duration.millis(timeInMillis),
                new KeyValue(sc.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                new KeyValue(sc.scaleXProperty(), sc.rectangle.scaleXProperty().get()),
                new KeyValue(sc.scaleYProperty(), sc.rectangle.scaleYProperty().get())
        );
        timeInMillis += 10;
        KeyFrame kf101 = new KeyFrame(Duration.millis(timeInMillis),
                new KeyValue(sc.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                new KeyValue(sc.scaleXProperty(), 1.5),
                new KeyValue(sc.scaleYProperty(), 1.5)
        );

        timeInMillis += 900;
        KeyFrame kf103 = new KeyFrame(Duration.millis(timeInMillis),
                new KeyValue(sc.scaleXProperty(), 1.5),
                new KeyValue(sc.scaleYProperty(), 1.5),
                new KeyValue(sc.rectangle.fillProperty(), Color.CORNFLOWERBLUE)
        );
        timeInMillis += 2000;
        KeyFrame kf102 = new KeyFrame(Duration.millis(timeInMillis),
                new KeyValue(sc.rectangle.fillProperty(), Color.rgb(253, 188, 66)),
                new KeyValue(sc.scaleXProperty(), 1.0),
                new KeyValue(sc.scaleYProperty(), 1.0)
        );

        tl.getKeyFrames().addAll(kf99, kf100, kf101, kf102, kf103);
        return tl;
    }

    public String getTextOfCell(int round, int cellOffset) {
        return cells[round][cellOffset].text.getText();
    }

    private class KeyCell extends StackPane {

        @FXML
        Text text;

        @FXML
        Rectangle rectangle;

        public KeyCell(String text) {
            this.setMinSize(0, 0);
            this.text = new Text(text);
            rectangle = new Rectangle();
            rectangle.widthProperty().bind(getInstance().widthProperty().divide(18));
            rectangle.heightProperty().bind(getInstance().widthProperty().divide(18));
            rectangle.setArcHeight(5);
            rectangle.setArcWidth(5);
            rectangle.setFill(Color.YELLOW);
            rectangle.setStroke(Color.BLACK);
            this.getChildren().addAll(rectangle, this.text);
        }

        public void setText(String text) {
            this.text.setText(text);
        }

        public String getText() {
            return this.text.getText();
        }
    }

    private class LabelCell extends StackPane {

        @FXML
        Text text;

        @FXML
        Rectangle rectangle;

        public LabelCell(String text) {
            this.text = new Text(text);
            this.text.setTextAlignment(TextAlignment.CENTER);
            setMinSize(0, 0);
            setMaxSize(Double.MAX_VALUE, Double.MAX_VALUE);
            rectangle = new Rectangle();
            rectangle.widthProperty().bind(getInstance().widthProperty().divide(18));
            rectangle.heightProperty().bind(getInstance().widthProperty().divide(18));
            rectangle.setVisible(false);
            this.getChildren().addAll(rectangle, this.text);
        }

        public void setText(String text) {
            this.text.setText(text);
        }

        public String getText() {
            return this.text.getText();
        }
    }
}
