/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package customControls;

import java.io.UnsupportedEncodingException;
import java.util.Random;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

/**
 *
 * @author Martin
 */
public class KeyInputControl extends HBox {

    private static final String[][] characters = new String[][]{
        {"!", "\"", "#", "$", "%", "&", "\'", "(", ")", "*", "+", ",", "-", ".", "/", "0", "1", "2", "3"},
        {"4", "5", "6", "7", "8", "9", ":", ";", "<", "=", ">", "?", "@", "A", "B", "C", "D", "E", "F"},
        {"G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y"},
        {"Z", "[", "\\", "]", "^", "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l"},
        {"m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~", " "},};

    @FXML
    HBox keyFieldContainer;

    @FXML
    Button addKey;

    @FXML
    Button generateKey;

    private KeyInputControl inputFields;

    int offset;

    public KeyInputControl() {
        VBox.setVgrow(this, Priority.ALWAYS);
        GridPane.setVgrow(this, Priority.NEVER);
        initialization();
        setHgrow(keyFieldContainer, Priority.ALWAYS);
        this.minHeightProperty().bind(((Region) keyFieldContainer.getChildren().get(0)).widthProperty());
        offset = 0;
        setAlignment(Pos.CENTER);
    }

    private void initialization() {
        keyFieldContainer = new HBox();
        keyFieldContainer.maxWidth(Double.MAX_VALUE);
        keyFieldContainer.maxHeight(Double.MAX_VALUE);
        for (int i = 0; i < 16; i++) {
            KeyCharacterCell kc = new KeyCharacterCell("", Color.WHITESMOKE);
            kc.setId(Integer.toString(i));
            keyFieldContainer.getChildren().add(kc);
            HBox.setHgrow(kc, Priority.ALWAYS);
            keyFieldContainer.setAlignment(Pos.CENTER_LEFT);
        }
        addKey = new Button("add key");
        addKey.setMaxWidth(Double.MAX_VALUE);
        addKey.addEventHandler(ActionEvent.ACTION, new javafx.event.EventHandler<ActionEvent>() {

            @Override
            public void handle(ActionEvent event) {
                KeyInputKeyboardControl kc = new KeyInputKeyboardControl(KeyInputControl.this);
                Parent root1 = new AnchorPane(kc);
                Stage stage = new Stage();
                stage.initModality(Modality.APPLICATION_MODAL);
                stage.initStyle(StageStyle.UNDECORATED);
                stage.setResizable(false);
                stage.setScene(new Scene(root1));
                stage.show();
            }
        });
        generateKey = new Button("generate key");
        generateKey.setMaxWidth(Double.MAX_VALUE);
        generateKey.addEventHandler(ActionEvent.ACTION, new javafx.event.EventHandler<ActionEvent>() {

            @Override
            public void handle(ActionEvent event) {
                Random random = new Random();
                for (int i = 0; i < 16; i++) {
                    int cislo = Math.abs(random.nextInt() % 95);
                    setCharacterToCell(i, characters[(int) Math.floor((double) cislo / 19)][cislo % 19]);
                }
            }
        });
        HBox.setMargin(addKey, new Insets(0, 0, 0, 5));
        HBox.setMargin(generateKey, new Insets(0, 0, 0, 5));
        HBox.setHgrow(generateKey, Priority.NEVER);
        HBox.setHgrow(addKey, Priority.NEVER);
        getChildren().add(keyFieldContainer);
        getChildren().add(addKey);
        getChildren().add(generateKey);
        keyFieldContainer.setMaxWidth(Double.MAX_VALUE);
        HBox.setHgrow(keyFieldContainer, Priority.ALWAYS);
    }

    public byte[] getKey() throws UnsupportedEncodingException {
        byte[] key = new byte[16];
        for (int i = 0; i < 16; i++) {
            byte[] bytes = ((KeyCharacterCell) keyFieldContainer.getChildren().get(i)).text.getText().getBytes("UTF-8");
            key[i] = bytes[0];
        }
        return key;
    }

    public void setCharacterToCell(int position, String text) {
        KeyCharacterCell kc = (KeyCharacterCell) keyFieldContainer.getChildren().get(position);
        kc.text.setText(text);
        kc.rectangle.setStroke(Color.CHARTREUSE);
    }

    ;

    public void setActiveCell(int position) {
        KeyCharacterCell kc = (KeyCharacterCell) keyFieldContainer.getChildren().get(position);
        kc.rectangle.setStroke(Color.CORNFLOWERBLUE);
    }

    private class KeyCharacterCell extends StackPane {

        @FXML
        Label text;

        @FXML
        Rectangle rectangle;

        private char character;

        public KeyCharacterCell(String text, Color color) {
            setMinSize(0, 0);
            this.text = new Label(text);
            HBox.setHgrow(this, Priority.ALWAYS);
            rectangle = new Rectangle();
            rectangle.widthProperty().bind(keyFieldContainer.widthProperty().get() <= keyFieldContainer.heightProperty().get() ? keyFieldContainer.widthProperty().divide(16).subtract(5) : keyFieldContainer.heightProperty().divide(16).subtract(5));
            rectangle.heightProperty().bind(keyFieldContainer.widthProperty().get() <= keyFieldContainer.heightProperty().get() ? keyFieldContainer.widthProperty().divide(16).subtract(5) : keyFieldContainer.heightProperty().divide(16).subtract(5));
            rectangle.setArcHeight(10);
            rectangle.setArcWidth(10);
            rectangle.setFill(color);
            rectangle.setStrokeWidth(3);
            rectangle.setStroke(Color.RED);
            getChildren().addAll(rectangle, this.text);
        }

        public void setText(String text) {
            this.text.setText(text);
        }

        public String getText() {
            return this.text.getText();
        }

        public char getCharacter() {
            return character;
        }

        public void setCharacter(char character) {
            this.character = character;
        }
    }
}
