/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package customControls;

import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Cursor;
import javafx.scene.effect.InnerShadow;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.Border;
import javafx.scene.layout.BorderStroke;
import javafx.scene.layout.BorderStrokeStyle;
import javafx.scene.layout.BorderWidths;
import javafx.scene.layout.CornerRadii;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import javafx.util.Duration;

/**
 *
 * @author Martin
 */
public class KeyInputKeyboardControl extends AnchorPane {

    private static final String[][] characters = new String[][]{
        {"!", "\"", "#", "$", "%", "&", "\'", "(", ")", "*", "+", ",", "-", ".", "/", "0", "1", "2", "3"},
        {"4", "5", "6", "7", "8", "9", ":", ";", "<", "=", ">", "?", "@", "A", "B", "C", "D", "E", "F"},
        {"G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y"},
        {"Z", "[", "\\", "]", "^", "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l"},
        {"m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~", " "},};

    @FXML
    HBox keyFieldContainer;

    private KeyInputControl keyFields;

    private int offset;

    public KeyInputKeyboardControl() {
        initialization();
    }

    public KeyInputKeyboardControl(KeyInputControl keyFields) {
        this.keyFields = keyFields;
        offset = 0;
        keyFields.setActiveCell(offset);
        setBorder(new Border(new BorderStroke(Color.BLACK, BorderStrokeStyle.SOLID, new CornerRadii(10), new BorderWidths(2))));
        initialization();
    }

    private void initialization() {
        VBox rows = new VBox(4);
        rows.setAlignment(Pos.CENTER);
        rows.setPadding(new Insets(10, 6, 6, 10));
        for (int i = 0; i < 5; i++) {
            HBox row = new HBox(4);
            row.setAlignment(Pos.CENTER);
            for (int j = 0; j < 19; j++) {
                KeyCharacterCell kc = new KeyCharacterCell(characters[i][j], Color.WHITESMOKE);
                kc.setId(Integer.toString(i) + Integer.toString(j));
                row.getChildren().add(kc);
            }
            rows.getChildren().add(row);
        }
        getChildren().add(rows);
    }

    private class KeyCharacterCell extends StackPane {

        @FXML
        Text text;

        @FXML
        Rectangle rectangle;

        private char character;

        public KeyCharacterCell(String text, Color color) {
            this.text = new Text(text);
            rectangle = new Rectangle(25, 25);
            rectangle.setArcHeight(10);
            rectangle.setArcWidth(10);
            rectangle.setFill(color);
            rectangle.setStrokeWidth(1);
            rectangle.setStroke(Color.DARKGREY);
            getChildren().addAll(rectangle, this.text);
            this.addEventHandler(MouseEvent.MOUSE_ENTERED, new javafx.event.EventHandler<MouseEvent>() {

                @Override
                public void handle(MouseEvent event) {
                    rectangle.setFill(color.darker());
                    getScene().setCursor(Cursor.HAND);
                }
            });
            this.addEventHandler(MouseEvent.MOUSE_EXITED, new javafx.event.EventHandler<MouseEvent>() {

                @Override
                public void handle(MouseEvent event) {
                    rectangle.setFill(color);
                    getScene().setCursor(Cursor.DEFAULT);
                }
            });
            this.addEventHandler(MouseEvent.MOUSE_CLICKED, new javafx.event.EventHandler<MouseEvent>() {

                @Override
                public void handle(MouseEvent event) {
                    rectangle.setFill(color);
                    InnerShadow is = new InnerShadow(8, 0, 0, Color.DARKGREY.darker());
                    final Timeline timeline = new Timeline();
                    final KeyValue kv = new KeyValue(rectangle.effectProperty(), is);
                    final KeyValue kv2 = new KeyValue(rectangle.strokeProperty(), Color.DARKGREY.darker());
                    final KeyValue kv3 = new KeyValue(rectangle.fillProperty(), color);
                    final KeyValue kv4 = new KeyValue(rectangle.strokeProperty(), Color.DARKGREY);
                    final KeyFrame kf = new KeyFrame(Duration.millis(150), kv);
                    final KeyFrame kf2 = new KeyFrame(Duration.millis(150), kv2);
                    final KeyFrame kf3 = new KeyFrame(Duration.millis(150), kv3);
                    final KeyValue kv1 = new KeyValue(rectangle.effectProperty(), null);
                    final KeyFrame kf1 = new KeyFrame(Duration.millis(400), kv1);
                    final KeyFrame kf4 = new KeyFrame(Duration.millis(400), kv4);
                    timeline.getKeyFrames().add(kf);
                    timeline.getKeyFrames().add(kf1);
                    timeline.getKeyFrames().add(kf2);
                    timeline.getKeyFrames().add(kf3);
                    timeline.getKeyFrames().add(kf4);
                    timeline.setCycleCount(1);
                    timeline.setAutoReverse(true);
                    keyFields.setCharacterToCell(offset++, text);
                    timeline.setOnFinished(new EventHandler<ActionEvent>() {

                        @Override
                        public void handle(ActionEvent event) {
                            if (offset >= 16) {
                                Stage stage = (Stage) getScene().getWindow();
                                stage.close();
                            } else {
                                keyFields.setActiveCell(offset);
                            }
                        }
                    });
                    timeline.play();
                    if (offset < 15) {
                        keyFields.setActiveCell(offset);
                    }
                }
            });
        }

        public void setText(String text) {
            this.text.setText(text);
        }

        public String getText() {
            return this.text.getText();
        }

        public char getCharacter() {
            return character;
        }

        public void setCharacter(char character) {
            this.character = character;
        }
    }
}
