package customControls;

import java.awt.image.BufferedImage;
import javafx.embed.swing.SwingFXUtils;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.scene.text.TextFlow;
import org.scilab.forge.jlatexmath.TeXConstants;
import org.scilab.forge.jlatexmath.TeXFormula;

public class MiddleTextBlock extends VBox {

    private static String mixColumnsMatrix = ("\\[\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\begin{bmatrix}\n"
            + " 02  &03  &01  &01  \\vphantom{S_{0,3}}\\\\\n"
            + " 01  &02  &03  &01  \\vphantom{S_{0,3}}\\\\\n"
            + " 01  &01  &02  &03  \\vphantom{S_{0,3}}\\\\\n"
            + " 03  &01  &01  &02  \\vphantom{S_{0,3}}\n"
            + "\\end{bmatrix}\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "=\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S`_{0,0}  &S`_{0,1}  &S`_{0,2}  &S`_{0,3}  \\\\\n"
            + " S`_{1,0}  &S`_{1,1}  &S`_{1,2}  &S`_{1,3}  \\\\\n"
            + " S`_{2,0}  &S`_{2,1}  &S`_{2,2}  &S`_{2,3}  \\\\\n"
            + " S`_{3,0}  &S`_{3,1}  &S`_{3,2}  &S`_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\]");

    private static String inverseMixColumnsMatrix = ("\\[\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\begin{bmatrix}\n"
            + " \\text{0E}  &\\text{0B}  &\\text{0D}  &\\text{09}  \\vphantom{S_{0,3}}\\\\\n"
            + " \\text{09}  &\\text{0E}  &\\text{0B}  &\\text{0D}  \\vphantom{S_{0,3}}\\\\\n"
            + " \\text{0D}  &\\text{09}  &\\text{0E}  &\\text{0B}  \\vphantom{S_{0,3}}\\\\\n"
            + " \\text{0B}  &\\text{0D}  &\\text{09}  &\\text{0E}  \\vphantom{S_{0,3}}\n"
            + "\\end{bmatrix}\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "=\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S`_{0,0}  &S`_{0,1}  &S`_{0,2}  &S`_{0,3}  \\\\\n"
            + " S`_{1,0}  &S`_{1,1}  &S`_{1,2}  &S`_{1,3}  \\\\\n"
            + " S`_{2,0}  &S`_{2,1}  &S`_{2,2}  &S`_{2,3}  \\\\\n"
            + " S`_{3,0}  &S`_{3,1}  &S`_{3,2}  &S`_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\]");

    private static String substituteBytesMatrix = ("\\[\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\rightarrow\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " SB_{x,y}  &SB_{x,y}  &SB_{x,y}  &SB_{x,y}  \\\\\n"
            + " SB_{x,y}  &SB_{x,y}  &SB_{x,y}  &SB_{x,y}  \\\\\n"
            + " SB_{x,y}  &SB_{x,y}  &SB_{x,y}  &SB_{x,y}  \\\\\n"
            + " SB_{x,y}  &SB_{x,y}  &SB_{x,y}  &SB_{x,y}  \n"
            + "\\end{bmatrix}}}}\\\\\n"
            + "\\vspace{15mm}\n"
            + "\\begin{align}"
            + "x &\\textit{ stands for counted SBox`s row index}\\\\\n"
            + "y &\\textit{ stands for counted SBox`s column index}\n"
            + "\\end{align}\\\\\n"
            + "\\]");

    private static String inverseSubstituteBytesMatrix = ("\\[\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\rightarrow\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  \\\\\n"
            + " ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  \\\\\n"
            + " ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  \\\\\n"
            + " ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  &ISB_{x,y}  \n"
            + "\\end{bmatrix}}}}\\\\\n"
            + "\\vspace{15mm}\n"
            + "\\begin{align}"
            + "x &\\textit{ stands for counted Inverse SBox`s row index}\\\\\n"
            + "y &\\textit{ stands for counted Inverse SBox`s column index}\n"
            + "\\end{align}\\\\\n"
            + "\\]");

    private static String shiftRowsMatrix = ("\\[\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\rightarrow\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,1}  &S_{1,2}  &S_{1,3}  &S_{1,0}  \\\\\n"
            + " S_{2,2}  &S_{2,3}  &S_{2,0}  &S_{2,1}  \\\\\n"
            + " S_{3,3}  &S_{3,0}  &S_{3,1}  &S_{3,2}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\]");

    private static String inverseShiftRowsMatrix = ("\\[\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\rightarrow\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,3}  &S_{1,0}  &S_{1,1}  &S_{1,2}  \\\\\n"
            + " S_{2,2}  &S_{2,3}  &S_{2,0}  &S_{2,1}  \\\\\n"
            + " S_{3,1}  &S_{3,2}  &S_{3,3}  &S_{3,0}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\]");

    private static String mixColumnsEquations = ("\\begin{align}\n"
            + "S`_{0,j} &= (2 \\cdot S_{0,j}) \\oplus (3 \\cdot S_{1,j}) \\oplus S_{2,j} \\oplus S_{3,j}\\\\\n "
            + "S`_{1,j} &= S_{0,j} \\oplus (2 \\cdot S_{1,j}) \\oplus (3 \\cdot S_{2,j}) \\oplus S_{3,j}\\\\\n "
            + "S`_{2,j} &= S_{0,j} \\oplus S_{1,j} \\oplus (2 \\cdot S_{2,j}) \\oplus (3 \\cdot S_{3,j})\\\\\n "
            + "S`_{3,j} &= (3 \\cdot S_{0,j}) \\oplus S_{1,j} \\oplus S_{2,j} \\oplus (2 \\cdot S_{3,j})\\\\\n "
            + "\\end{align}\\\\\n"
            + "\\vspace{15mm}"
            + "\\begin{align}"
            + "\\oplus &\\textit{ is indicating bitwise XOR}\\\\ \\cdot &\\textit{ is indicating multiplication in GF(2^8)}\\\\\n"
            + "\\end{align}\\\\\n");

    private static String inverseMixColumnsEquations = ("\\begin{align}\n"
            + "S`_{0,j} &= (\\text{0E} \\cdot S_{0,j}) \\oplus (\\text{0B} \\cdot S_{1,j}) \\oplus (\\text{0D} \\cdot S_{2,j}) \\oplus (\\text{09} \\cdot S_{3,j})\\\\\n "
            + "S`_{1,j} &= (\\text{09} \\cdot S_{0,j}) \\oplus (\\text{0E} \\cdot S_{1,j}) \\oplus (\\text{0B} \\cdot S_{2,j}) \\oplus (\\text{0D} \\cdot S_{3,j})\\\\\n "
            + "S`_{2,j} &= (\\text{0D} \\cdot S_{0,j}) \\oplus (\\text{09} \\cdot S_{1,j}) \\oplus (\\text{0E} \\cdot S_{2,j}) \\oplus (\\text{0B} \\cdot S_{3,j})\\\\\n "
            + "S`_{3,j} &= (\\text{0B} \\cdot S_{0,j}) \\oplus (\\text{0D} \\cdot S_{1,j}) \\oplus (\\text{09} \\cdot S_{2,j}) \\oplus (\\text{0E} \\cdot S_{3,j})\\\\\n "
            + "\\end{align}\\\\\n"
            + "\\vspace{15mm}"
            + "\\begin{align}"
            + "\\oplus &\\textit{ is indicating bitwise XOR}\\\\ \\cdot &\\textit{ is indicating multiplication in GF(2^8)}\\\\\n"
            + "\\end{align}\\\\\n");

    private static String addRoundKeyMatrix = (""
            + "\\definecolor{yelloww}{rgb}{1,1,0}\n"
            + "\\definecolor{yellowgreenn}{rgb}{0.6,0.8,0.2}\n"
            + "\\definecolor{redd}{rgb}{1,0,0}\n"
            + "\\[\n"
            + "\\textcolor{redd}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S_{0,0}  &S_{0,1}  &S_{0,2}  &S_{0,3}  \\\\\n"
            + " S_{1,0}  &S_{1,1}  &S_{1,2}  &S_{1,3}  \\\\\n"
            + " S_{2,0}  &S_{2,1}  &S_{2,2}  &S_{2,3}  \\\\\n"
            + " S_{3,0}  &S_{3,1}  &S_{3,2}  &S_{3,3}  \n"
            + "\\end{bmatrix}}}}\n"
            + "\\oplus\n"
            + "\\textcolor{dandelion}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " w_{i}  &w_{i+1}  &w_{i+2}  &w_{i+3} \\vphantom{S_{0,3}} \\\\\n"
            + " w_{i}  &w_{i+1}  &w_{i+2}  &w_{i+3} \\vphantom{S_{0,3}} \\\\\n"
            + " w_{i}  &w_{i+1}  &w_{i+2}  &w_{i+3} \\vphantom{S_{0,3}} \\\\\n"
            + " w_{i}  &w_{i+1}  &w_{i+2}  &w_{i+3} \\vphantom{S_{0,3}} \n"
            + "\\end{bmatrix}}}}\n"
            + "=\n"
            + "\\textcolor{yellowgreenn}{\\textbf{\\textit{\n"
            + "\\begin{bmatrix}\n"
            + " S`_{0,0}  &S`_{0,1}  &S`_{0,2}  &S`_{0,3}  \\\\\n"
            + " S`_{1,0}  &S`_{1,1}  &S`_{1,2}  &S`_{1,3}  \\\\\n"
            + " S`_{2,0}  &S`_{2,1}  &S`_{2,2}  &S`_{2,3}  \\\\\n"
            + " S`_{3,0}  &S`_{3,1}  &S`_{3,2}  &S`_{3,3}  \n"
            + "\\end{bmatrix}}}}\\\\\n"
            + "\\vspace{15mm}\n"
            + "\\begin{align}"
            + "\\oplus &\\textit{ is indicating bitwise XOR}\\\\\n"
            + "w &\\textit{ is expanded key word}\n"
            + "\\end{align}\\\\\n"
            + "\\]");

    private Label headline;

    private typeOfDescription type;

    public MiddleTextBlock() {
        setPadding(new Insets(10, 20, 0, 20));
        setSpacing(20);
        headline = new Label();
        headline.setFont(new Font(22));
        headline.setTextFill(Color.ORCHID);
        getChildren().add(headline);
    }

    public void setType(typeOfDescription type) {
        this.type = type;
        switch(type) {
            
            case ADD_ROUND_KEY:
                getChildren().clear();
                headline.setText("Add round key operation");
                Text t = new Text("Bytes of StateArray Matrix are XORed with corresponding bytes of expanded key words. The exact word is choosen according to the byte location inside the StateArray and the current number of the round. Iverse version is the same only the words are taken with the opposite order.");
                t.setFont(new Font(14));
                TextFlow textFlow = new TextFlow(t);
                TeXFormula tex = new TeXFormula(addRoundKeyMatrix);
                StackPane s = new StackPane();
                s.setAlignment(Pos.CENTER);
                java.awt.Image awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                Image fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                ImageView view = new ImageView(fxImage);
                s.getChildren().add(view);
                getChildren().addAll(headline, textFlow, s);
                break;
                
            case SHIFT_ROWS:
                getChildren().clear();
                headline.setText("Shift rows operation");
                t = new Text("StateArray Matrix is altered as follows:\n\n" + " - first row remains untouched\n" + " - second row 1-byte circular left shifted\n" + " - third row 2-byte circular left shifted\n" + " - fourth row 3-byte circular left shifted");
                t.setFont(new Font(14));
                textFlow = new TextFlow(t);
                tex = new TeXFormula(shiftRowsMatrix);
                awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                view = new ImageView(fxImage);
                s = new StackPane(view);
                s.setAlignment(Pos.CENTER);
                getChildren().addAll(headline, textFlow, s);
                break;
                
            case SUBSTITUTE_BYTES:
                getChildren().clear();
                t = new Text("Bytes of the StateArray Matrix are substituted with the bytes of the SBox. The location inside the SBox is counted from the substituted byte. First half of the byte represents Sbox`s row index and the second half represents the sbox`s column index.");
                headline.setText("Substitute bytes operation");
                t.setFont(new Font(14));
                textFlow = new TextFlow(t);
                s = new StackPane();
                s.setAlignment(Pos.CENTER);
                tex = new TeXFormula(substituteBytesMatrix);
                awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                view = new ImageView(fxImage);
                s.getChildren().add(view);
                getChildren().addAll(headline, textFlow, s);
                break;
                
            case INVERSE_SHIFT_ROWS:
                getChildren().clear();
                headline.setText("Inverse shift rows operation");
                t = new Text("Iverse Shift rows operation is the same as Shift rows operation only right circular byte shift is now performed. StateArray Matrix is altered as follows:\n\n" + " - first row remains untouched\n" + " - second row 1-byte circular right shifted\n" + " - third row 2-byte circular right shifted\n" + " - fourth row 3-byte circular tight shifted");
                t.setFont(new Font(14));
                textFlow = new TextFlow(t);
                tex = new TeXFormula(inverseShiftRowsMatrix);
                awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                view = new ImageView(fxImage);
                s = new StackPane(view);
                s.setAlignment(Pos.CENTER);
                getChildren().addAll(headline, textFlow, s);
                break;
                
            case INVERSE_SUBTITUTE_BYTES:
                getChildren().clear();
                t = new Text("Bytes of the StateArray Matrix are substituted with the bytes of the Inverse SBox. The location inside the Inverse SBox is counted from the substituted byte. First half of the byte represents Inverse Sbox`s row index and the second half represents the Inverse sbox`s column index.");
                headline.setText("Substitute bytes operation");
                t.setFont(new Font(14));
                textFlow = new TextFlow(t);
                s = new StackPane();
                s.setAlignment(Pos.CENTER);
                tex = new TeXFormula(inverseSubstituteBytesMatrix);
                awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                view = new ImageView(fxImage);
                s.getChildren().add(view);
                getChildren().addAll(headline, textFlow, s);
                break;
                
            case MIX_COLUMNS:
                getChildren().clear();
                headline.setText("Mix columns operation");
                t = new Text("This operation is altering each column of the StateArray Matrix individually. Each byte of the new column is function of all four old bytes of that column. The function is described with the Matrix multiplication below.");
                t.setFont(new Font(14));
                textFlow = new TextFlow(t);
                tex = new TeXFormula(mixColumnsMatrix);
                awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                view = new ImageView(fxImage);
                s = new StackPane(view);
                s.setAlignment(Pos.CENTER);
                TeXFormula tex2 = new TeXFormula(mixColumnsEquations);
                java.awt.Image awtImage2 = tex2.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                Image fxImage2 = SwingFXUtils.toFXImage((BufferedImage) awtImage2, null);
                ImageView view2 = new ImageView(fxImage2);
                StackPane s2 = new StackPane(view2);
                Text t2 = new Text("MixColumns transformation on one column can be expressed from the equation as:");
                t2.setFont(new Font(14));
                TextFlow textFlow2 = new TextFlow(t2);
                s2.setAlignment(Pos.CENTER);
                getChildren().addAll(headline, textFlow, s, textFlow2, s2);
                break;
                
            case INVERSE_MIXCOLUMNS:
                getChildren().clear();
                headline.setText("Inverse mix columns operation");
                t = new Text("This operation is altering each column of the StateArray Matrix individually. Each byte of the new column is function of all four old bytes of that column. The function is described with the Matrix multiplication below.");
                t.setFont(new Font(14));
                textFlow = new TextFlow(t);
                tex = new TeXFormula(inverseMixColumnsMatrix);
                awtImage = tex.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage = SwingFXUtils.toFXImage((BufferedImage) awtImage, null);
                view = new ImageView(fxImage);
                s = new StackPane(view);
                s.setAlignment(Pos.CENTER);
                tex2 = new TeXFormula(inverseMixColumnsEquations);
                awtImage2 = tex2.createBufferedImage(TeXConstants.SCRIPT_NORMAL, 17, java.awt.Color.DARK_GRAY, java.awt.Color.WHITE);
                fxImage2 = SwingFXUtils.toFXImage((BufferedImage) awtImage2, null);
                view2 = new ImageView(fxImage2);
                s2 = new StackPane(view2);
                t2 = new Text("Inverse MixColumns transformation on one column can be expressed from the equation as:");
                t2.setFont(new Font(14));
                textFlow2 = new TextFlow(t2);
                s2.setAlignment(Pos.CENTER);
                getChildren().addAll(headline, textFlow, s, textFlow2, s2);
                break;
        }
    }

    public enum typeOfDescription {

        SHIFT_ROWS, INVERSE_SHIFT_ROWS, MIX_COLUMNS, INVERSE_MIXCOLUMNS, ADD_ROUND_KEY, SUBSTITUTE_BYTES, INVERSE_SUBTITUTE_BYTES
    }
}
