package customControls;

import java.util.Observable;
import javafx.beans.InvalidationListener;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Tooltip;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.RowConstraints;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;

public class PlainTextCipherTextControl extends GridPane {

    private int index;

    private int activeIndex;

    private byte[] input;

    private byte[] output;

    private Label startLabel;

    private Label endLabel2;

    private Label endLabel;

    private Label startLabel2;

    private Label plainTextLabel;

    private Label cipherTextLabel;

    private HBox inputCellsContainer;

    private HBox outputCellsContainer;

    private HBox labelHBox;

    private Button inputOutputNextButton;

    private Button inputOutputPreviousButton;

    private StateArrayControl stateArray;

    private SimpleBooleanProperty indexPreviousButtonProperty;

    private SimpleBooleanProperty indexNextButtonProperty;

    private boolean encryption;

    private static Font toolTipFont = new Font(12);

    public PlainTextCipherTextControl() {
        encryption = true;
        setPadding(new Insets(5));
        buttonInitialization();
        index = 0;
        activeIndex = 0;
        ColumnConstraints cc0 = new ColumnConstraints();
        cc0.setMaxWidth(Double.MAX_VALUE);
        cc0.setHgrow(Priority.NEVER);
        cc0.setMinWidth(140);
        cc0.setMaxWidth(140);
        ColumnConstraints cc1 = new ColumnConstraints();
        cc1.setMaxWidth(Double.MAX_VALUE);
        cc1.setHgrow(Priority.NEVER);
        ColumnConstraints cc2 = new ColumnConstraints();
        cc2.setMaxWidth(Double.MAX_VALUE);
        cc2.setHgrow(Priority.ALWAYS);
        ColumnConstraints cc3 = new ColumnConstraints();
        cc3.setMaxWidth(Double.MAX_VALUE);
        cc3.setHgrow(Priority.NEVER);
        RowConstraints rc0 = new RowConstraints();
        rc0.setVgrow(Priority.ALWAYS);
        rc0.setFillHeight(false);
        rc0.setMaxHeight(Double.MAX_VALUE);
        rc0.maxHeightProperty().bind(((Region) inputCellsContainer.getChildren().get(0)).widthProperty().add(8));
        RowConstraints rc1 = new RowConstraints();
        rc1.setVgrow(Priority.ALWAYS);
        rc1.setFillHeight(false);
        rc1.setMaxHeight(Double.MAX_VALUE);
        rc1.maxHeightProperty().bind(((Region) outputCellsContainer.getChildren().get(0)).widthProperty().add(8));
        RowConstraints rc2 = new RowConstraints();
        rc2.setVgrow(Priority.NEVER);
        getColumnConstraints().addAll(cc0, cc1, cc2, cc3);
        getRowConstraints().addAll(rc0, rc1, rc2);
        widthProperty().addListener(new InvalidationListener() {

            @Override
            public void invalidated(javafx.beans.Observable observable) {
                layout();
            }
        });
        heightProperty().addListener(new InvalidationListener() {

            public void invalidated(Observable observable) {
            }

            @Override
            public void invalidated(javafx.beans.Observable observable) {
                layout();
            }
        });
    }

    public void swithToDecrEncr() {
        if (encryption) {
            encryption = false;
            byte[] input1 = input;
            input = output;
            output = input1;
            fillPlainText(0);
            plainTextLabel.setText("cipherText: ");
            cipherTextLabel.setText("plainText: ");
        } else {
            encryption = true;
            byte[] input1 = input;
            input = output;
            output = input1;
            fillPlainText(0);
            plainTextLabel.setText("plainText: ");
            cipherTextLabel.setText("cipherText: ");
        }
    }

    public byte[] getInput() {
        return input;
    }

    public byte[] getOutput() {
        return output;
    }

    public void setInput(byte[] input) {
        this.input = input;
        index = 0;
        activeIndex = 0;
        indexPreviousButtonProperty = new SimpleBooleanProperty(activeIndex == 0);
        inputOutputPreviousButton.disableProperty().bind(indexPreviousButtonProperty);
        indexNextButtonProperty = new SimpleBooleanProperty((((input.length / 16) - 1) == activeIndex));
        inputOutputNextButton.disableProperty().bind(indexNextButtonProperty);
    }

    public void setOutput(byte[] output) {
        this.output = output;
    }

    public void setStateArray(StateArrayControl stateArray) {
        this.stateArray = stateArray;
    }

    private void buttonInitialization() {
        labelHBox = new HBox(0);
        inputCellsContainer = new HBox(2);
        outputCellsContainer = new HBox(2);
        
        inputOutputPreviousButton = new Button("<<");
        Tooltip tp = new Tooltip("Previous block");
        tp.setFont(toolTipFont);
        inputOutputPreviousButton.setTooltip(tp);
        inputOutputPreviousButton.maxWidth(Double.MAX_VALUE);
        inputOutputPreviousButton.setMinWidth(40);
        GridPane.setHgrow(inputOutputPreviousButton, Priority.NEVER);
        GridPane.setVgrow(inputOutputPreviousButton, Priority.NEVER);
        GridPane.setMargin(inputOutputPreviousButton, new Insets(5));
        
        inputOutputNextButton = new Button(">>");
        Tooltip tn = new Tooltip("Next block");
        tn.setFont(toolTipFont);
        inputOutputNextButton.setTooltip(tn);
        inputOutputNextButton.maxWidth(Double.MAX_VALUE);
        inputOutputNextButton.setMinWidth(40);
        GridPane.setHgrow(inputOutputNextButton, Priority.NEVER);
        GridPane.setVgrow(inputOutputNextButton, Priority.NEVER);
        GridPane.setMargin(inputOutputNextButton, new Insets(5));
        
        for (int i = 0; i < 32; i++) {
            InputOutputCell cellIn = new InputOutputCell("", Color.AQUA, 0);
            InputOutputCell cellOut = new InputOutputCell("", Color.LIGHTSTEELBLUE, 1);
            HBox.setHgrow(cellIn, Priority.ALWAYS);
            HBox.setHgrow(cellOut, Priority.ALWAYS);
            if (i == 16) {
                HBox.setMargin(cellIn, new Insets(0, 0, 0, 5));
                HBox.setMargin(cellOut, new Insets(0, 0, 0, 5));
            }
            inputCellsContainer.getChildren().add(cellIn);
            outputCellsContainer.getChildren().add(cellOut);
        }
        GridPane.setHgrow(inputCellsContainer, Priority.ALWAYS);
        GridPane.setHgrow(outputCellsContainer, Priority.ALWAYS);
        GridPane.setVgrow(inputCellsContainer, Priority.ALWAYS);
        GridPane.setVgrow(outputCellsContainer, Priority.ALWAYS);
        GridPane.setHgrow(labelHBox, Priority.ALWAYS);
        
        inputOutputNextButton.addEventHandler(MouseEvent.MOUSE_CLICKED, new EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                fillPlainText(activeIndex + 1);
                indexNextButtonProperty.set((((input.length / 16) - 1) == activeIndex));
                indexPreviousButtonProperty.set(activeIndex == 0);
            }
        });
        inputOutputPreviousButton.addEventHandler(MouseEvent.MOUSE_CLICKED, new EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                fillPlainText(activeIndex - 1);
                indexNextButtonProperty.set((((input.length / 16) - 1) == activeIndex));
                indexPreviousButtonProperty.set(activeIndex == 0);
            }
        });
        Tooltip tl = new Tooltip("Last byte of block index");
        tl.setFont(toolTipFont);
        Tooltip tf = new Tooltip("First byte of block index");
        tf.setFont(toolTipFont);
        startLabel = new Label("0");
        startLabel.setTooltip(tf);
        StackPane sl1 = new StackPane(startLabel);
        StackPane.setAlignment(startLabel, Pos.CENTER_LEFT);
        endLabel = new Label("15");
        endLabel.setTooltip(tl);
        StackPane sl2 = new StackPane(endLabel);
        StackPane.setAlignment(endLabel, Pos.CENTER_RIGHT);
        startLabel2 = new Label("16");
        startLabel2.setTooltip(tf);
        StackPane sl3 = new StackPane(startLabel2);
        StackPane.setAlignment(startLabel2, Pos.CENTER_LEFT);
        HBox.setMargin(sl3, new Insets(0, 0, 0, 7));
        endLabel2 = new Label("31");
        endLabel2.setTooltip(tl);
        StackPane sl4 = new StackPane(endLabel2);
        StackPane.setAlignment(endLabel2, Pos.CENTER_RIGHT);
        HBox.setHgrow(sl1, Priority.ALWAYS);
        HBox.setHgrow(sl2, Priority.ALWAYS);
        HBox.setHgrow(sl3, Priority.ALWAYS);
        HBox.setHgrow(sl4, Priority.ALWAYS);
        startLabel.maxWidth(Double.MAX_VALUE);
        startLabel2.maxWidth(Double.MAX_VALUE);
        endLabel.maxWidth(Double.MAX_VALUE);
        endLabel2.maxWidth(Double.MAX_VALUE);
        startLabel.setTextAlignment(TextAlignment.LEFT);
        endLabel.setTextAlignment(TextAlignment.RIGHT);
        startLabel2.setTextAlignment(TextAlignment.LEFT);
        endLabel2.setTextAlignment(TextAlignment.RIGHT);
        labelHBox.getChildren().addAll(sl1, sl2, sl3, sl4);
        plainTextLabel = new Label("plainText: ");
        plainTextLabel.setTextFill(Color.AQUA);
        plainTextLabel.setFont(Font.font("system", FontWeight.BOLD, 20));
        plainTextLabel.setMinWidth(120);
        cipherTextLabel = new Label("cipherText: ");
        cipherTextLabel.setTextFill(Color.LIGHTSTEELBLUE);
        cipherTextLabel.setFont(Font.font("system", FontWeight.BOLD, 20));
        cipherTextLabel.setMinWidth(120);
        
        this.add(plainTextLabel, 0, 0);
        this.add(cipherTextLabel, 0, 1);
        this.add(inputOutputPreviousButton, 1, 0, 1, 2);
        this.add(inputCellsContainer, 2, 0);
        this.add(outputCellsContainer, 2, 1);
        this.add(inputOutputNextButton, 3, 0, 1, 2);
        this.add(labelHBox, 2, 2);
    }

    public int getIndex() {
        return index;
    }

    ;

    public void fillPlainText(int plainTextOffset) {
        boolean changed = false;
        int reminds = (input.length / 16) - plainTextOffset;
        if (reminds > 2) {
            reminds = 2;
        }
        if (plainTextOffset <= input.length / 16 && plainTextOffset >= 0 && input != null && output != null) {
            if (activeIndex == index && plainTextOffset > index && reminds > 0) {
                activeIndex++;
                changed = true;
            } else if (activeIndex > index && plainTextOffset < activeIndex) {
                activeIndex--;
                changed = true;
            } else if (reminds != 0) {
                if (index > plainTextOffset) {
                    activeIndex--;
                    changed = true;
                    index = plainTextOffset;
                } else if (plainTextOffset > index) {
                    index++;
                    activeIndex++;
                    changed = true;
                }
                if (reminds > 1 || activeIndex > index) {
                    for (int i = 0; i < 2; i++) {
                        for (int j = 0; j < 16; j++) {
                            InputOutputCell cell = (InputOutputCell) inputCellsContainer.getChildren().get(16 * i + j);
                            cell.setText(String.format("%02X", input[((index + i) * 16) + j]));
                            cell = (InputOutputCell) outputCellsContainer.getChildren().get(16 * i + j);
                            cell.setText(String.format("%02X", output[((index + i) * 16) + j]));
                        }
                    }
                }
                if (reminds == 1 && activeIndex == index) {
                    for (int i = 0; i < 16; i++) {
                        InputOutputCell cell = (InputOutputCell) inputCellsContainer.getChildren().get(16 + i);
                        cell.setText("");
                        cell = (InputOutputCell) outputCellsContainer.getChildren().get(16 + i);
                        cell.setText("");
                        cell = (InputOutputCell) inputCellsContainer.getChildren().get(i);
                        cell.setText(String.format("%02X", input[(index * 16 + i)]));
                        cell = (InputOutputCell) outputCellsContainer.getChildren().get(i);
                        cell.setText(String.format("%02X", output[(index * 16 + i)]));
                        startLabel2.setVisible(false);
                        endLabel2.setVisible(false);
                    }
                } else {
                    startLabel2.setVisible(true);
                    endLabel2.setVisible(true);
                }
            }
            startLabel.setText(String.valueOf(16 * index));
            endLabel.setText(String.valueOf(16 * index + 15));
            startLabel2.setText(String.valueOf(16 * index + 16));
            endLabel2.setText(String.valueOf(index * 16 + 31));
            if (changed) {
                stateArray.setBlockIndex(activeIndex);
            }
        }
        highlightActiveCells();
    }

    private void highlightActiveCells() {
        if (index == activeIndex) {
            for (int i = 0; i < 16; i++) {
                InputOutputCell cell = (InputOutputCell) inputCellsContainer.getChildren().get(i);
                cell.rectangle.setStroke(Color.CORNFLOWERBLUE);
                cell = (InputOutputCell) outputCellsContainer.getChildren().get(i);
                cell.rectangle.setStroke(Color.CORNFLOWERBLUE);
                cell = (InputOutputCell) inputCellsContainer.getChildren().get(i + 16);
                cell.rectangle.setStroke(Color.DARKGRAY);
                cell = (InputOutputCell) outputCellsContainer.getChildren().get(i + 16);
                cell.rectangle.setStroke(Color.DARKGRAY);
            }
        } else if (activeIndex > index) {
            for (int i = 0; i < 16; i++) {
                InputOutputCell cell = (InputOutputCell) inputCellsContainer.getChildren().get(i);
                cell.rectangle.setStroke(Color.DARKGRAY);
                cell = (InputOutputCell) outputCellsContainer.getChildren().get(i);
                cell.rectangle.setStroke(Color.DARKGRAY);
                cell = (InputOutputCell) inputCellsContainer.getChildren().get(i + 16);
                cell.rectangle.setStroke(Color.CORNFLOWERBLUE);
                cell = (InputOutputCell) outputCellsContainer.getChildren().get(i + 16);
                cell.rectangle.setStroke(Color.CORNFLOWERBLUE);
            }
        }
    }

    public SimpleBooleanProperty getIndexPreviousButtonProperty() {
        return indexPreviousButtonProperty;
    }

    public SimpleBooleanProperty getIndexNextButtonProperty() {
        return indexNextButtonProperty;
    }

    public Button getInputOutputNextButton() {
        return inputOutputNextButton;
    }

    public Button getInputOutputPreviousButton() {
        return inputOutputPreviousButton;
    }

    private class InputOutputCell extends StackPane {

        @FXML
        Text text;

        @FXML
        Rectangle rectangle;

        public InputOutputCell(String text, Color color, int type) {
            setMinSize(0, 0);
            this.text = new Text(text);
            this.text.setFont(Font.font("system", FontWeight.BOLD, USE_PREF_SIZE));
            rectangle = new Rectangle();
            this.widthProperty().addListener(new ChangeListener<Number>() {

                @Override
                public void changed(ObservableValue<? extends Number> observableValue, Number oldSceneHeight, Number newSceneHeight) {
                }
            });
            if (type == 0) {
                rectangle.widthProperty().bind(inputCellsContainer.widthProperty().divide(32).subtract(4));
                rectangle.heightProperty().bind(inputCellsContainer.widthProperty().divide(32).subtract(4));
            } else {
                rectangle.widthProperty().bind(outputCellsContainer.widthProperty().divide(32).subtract(4));
                rectangle.heightProperty().bind(outputCellsContainer.widthProperty().divide(32).subtract(4));
            }
            rectangle.maxWidth(Double.MAX_VALUE);
            rectangle.maxHeight(Double.MAX_VALUE);
            rectangle.setArcHeight(10);
            rectangle.setArcWidth(10);
            rectangle.setFill(color);
            rectangle.setStroke(Color.DARKGRAY);
            rectangle.setStrokeWidth(2);
            getChildren().addAll(rectangle, this.text);
        }

        public void setText(String text) {
            this.text.setText(text);
        }

        public String getText() {
            return this.text.getText();
        }
    }
}
