package customControls;

import aes.VisualizationVariablesContainer;
import javafx.animation.Animation;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.beans.InvalidationListener;
import javafx.beans.Observable;
import javafx.beans.binding.Bindings;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.HPos;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.geometry.VPos;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.Slider;
import javafx.scene.control.Tooltip;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.ColumnConstraints;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.Priority;
import javafx.scene.layout.Region;
import javafx.scene.layout.RowConstraints;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;
import javafx.scene.text.TextFlow;
import javafx.util.Duration;

public class StateArrayControl extends GridPane {

    private Button buttonPrevious;

    private Button buttonNext;

    private Button buttonStart;

    private Button buttonPause;

    private Button buttonStop;

    private Pane stateArrayCellsContainer;

    private Label roundNameLabel;

    private String phaseNames[];

    private TextFlow animationText;

    private SBoxInverseSBoxControl sbox;

    private VisualizationVariablesContainer data;

    private int phaseIndex;

    private int blockIndex;

    private StateArrayCell[][] cells;

    private SimpleBooleanProperty indexPropertyNextButton;

    private SimpleBooleanProperty indexPropertyPreviousButton;

    private KeyFieldsControl keyField;

    private boolean encryptionMode;

    private Timeline animation;

    private Slider phaseIndexSlider;

    private Label sliderStartLabel;

    private Label sliderEndLabel;

    private MiddleTextBlock middleText;

    private Label roundLabel;

    private Label roundNumberLabel;

    private SimpleStringProperty roundNumberStringProperty;

    private PlainTextCipherTextControl plainTextCipherTextControl;

    private SimpleBooleanProperty lastRoundProperty;

    private Label operationIndexLabel;

    private static Font toolTipFont = new Font(12);

    public StateArrayControl() {
        encryptionMode = true;
        creatingStackPanes();
        animation = new Timeline();
        setConstraints();
        Initialization();
        setPadding(new Insets(20, 0, 0, 0));
        GridPane.setHgrow(this, Priority.ALWAYS);
        GridPane.setVgrow(this, Priority.ALWAYS);
        buttonStart.addEventHandler(MouseEvent.MOUSE_CLICKED, new EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                if (animation.statusProperty().isEqualTo(Animation.Status.STOPPED).get()) {
                    int animIndex = (phaseIndex - 1) % 4;
                    if (phaseIndex == 0 || phaseIndex == 39) {
                        animation = addKeyAnimation();
                    } else if (phaseIndex == 40) {
                        animation = new Timeline();
                    } else if (encryptionMode) {
                        switch(animIndex) {
                            case 0:
                                animation = substiteBytesAnimation();
                                break;
                            case 1:
                                animation = swiftRowsAnim();
                                break;
                            case 2:
                                animation = mixColumnsAnimation();
                                break;
                            case 3:
                                animation = addKeyAnimation();
                        }
                    } else {
                        switch(animIndex) {
                            case 0:
                                animation = invereSwiftRowsAnim();
                                break;
                            case 1:
                                animation = substiteBytesAnimation();
                                break;
                            case 2:
                                animation = addKeyAnimation();
                                break;
                            case 3:
                                animation = inverseMixColumnsAnimation();
                        }
                    }
                    refreshAnimationButtonProperties();
                    animation.play();
                } else {
                    animation.play();
                }
            }
        });
        buttonPause.addEventHandler(MouseEvent.MOUSE_CLICKED, new EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                if (animation.statusProperty().isEqualTo(Animation.Status.RUNNING).get()) {
                    animation.pause();
                }
            }
        });
        buttonStop.addEventHandler(MouseEvent.MOUSE_CLICKED, new EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                if (animation.statusProperty().isEqualTo(Animation.Status.RUNNING).get() || animation.statusProperty().isEqualTo(Animation.Status.PAUSED).get()) {
                    animation.stop();
                    refresh();
                    if (encryptionMode) {
                        keyField.refresh(Integer.parseInt(roundNumberStringProperty.get()));
                    } else {
                        keyField.refresh(10 - Integer.parseInt(roundNumberStringProperty.get()));
                    }
                    sbox.refresh();
                }
            }
        });
    }

    private void refreshAnimationButtonProperties() {
        buttonNext.disableProperty().bind(Bindings.or(indexPropertyNextButton, animation.statusProperty()
                .isNotEqualTo(Animation.Status.STOPPED)));

        buttonPrevious.disableProperty().bind(Bindings.or(indexPropertyPreviousButton, animation.statusProperty()
                .isNotEqualTo(Animation.Status.STOPPED)));

        buttonStart.disableProperty().bind(Bindings.or(animation.statusProperty()
                .isEqualTo(Animation.Status.RUNNING), lastRoundProperty));
        
        buttonPause.disableProperty().bind(animation.statusProperty()
                .isNotEqualTo(Animation.Status.RUNNING));

        buttonStop.disableProperty().bind(animation.statusProperty()
                .isEqualTo(Animation.Status.STOPPED));

        phaseIndexSlider.disableProperty().bind(animation.statusProperty()
                .isNotEqualTo(Animation.Status.STOPPED));

        plainTextCipherTextControl.getInputOutputNextButton().disableProperty()
                .bind(Bindings.or(plainTextCipherTextControl.getIndexNextButtonProperty(), animation.statusProperty()
                .isNotEqualTo(Animation.Status.STOPPED)));

        plainTextCipherTextControl.getInputOutputPreviousButton().disableProperty()
                .bind(Bindings.or(plainTextCipherTextControl.getIndexPreviousButtonProperty(), animation.statusProperty()
                .isNotEqualTo(Animation.Status.STOPPED)));

    }

    public void setPlainTextCipherTextControl(PlainTextCipherTextControl plainTextCipherTextControl) {
        this.plainTextCipherTextControl = plainTextCipherTextControl;
    }

    public void setMiddleText(MiddleTextBlock middleText) {
        this.middleText = middleText;
    }

    public void setSBox(SBoxInverseSBoxControl sb) {
        sbox = sb;
    }

    public void setkeyField(KeyFieldsControl keyField) {
        this.keyField = keyField;
    }

    Timeline swiftRowsAnim() {
        refresh();
        Timeline anim = new Timeline();
        SimpleDoubleProperty cell10height = new SimpleDoubleProperty(cells[1][0].layoutYProperty().get());
        SimpleDoubleProperty cell10width = new SimpleDoubleProperty(cells[1][0].layoutXProperty().get());
        SimpleDoubleProperty cell11width = new SimpleDoubleProperty(cells[1][1].layoutXProperty().get());
        SimpleDoubleProperty cell12width = new SimpleDoubleProperty(cells[1][2].layoutXProperty().get());
        SimpleDoubleProperty cell13width = new SimpleDoubleProperty(cells[1][3].layoutXProperty().get());
        SimpleDoubleProperty cell20width = new SimpleDoubleProperty(cells[2][0].layoutXProperty().get());
        SimpleDoubleProperty cell21width = new SimpleDoubleProperty(cells[2][1].layoutXProperty().get());
        SimpleDoubleProperty cell22width = new SimpleDoubleProperty(cells[2][2].layoutXProperty().get());
        SimpleDoubleProperty cell23width = new SimpleDoubleProperty(cells[2][3].layoutXProperty().get());
        SimpleDoubleProperty cell20height = new SimpleDoubleProperty(cells[2][0].layoutYProperty().get());
        SimpleDoubleProperty cell21height = new SimpleDoubleProperty(cells[2][1].layoutYProperty().get());
        SimpleDoubleProperty cell33height = new SimpleDoubleProperty(cells[3][3].layoutYProperty().get());
        SimpleDoubleProperty cell30width = new SimpleDoubleProperty(cells[3][0].layoutXProperty().get());
        SimpleDoubleProperty cell31width = new SimpleDoubleProperty(cells[3][1].layoutXProperty().get());
        SimpleDoubleProperty cell32width = new SimpleDoubleProperty(cells[3][2].layoutXProperty().get());
        SimpleDoubleProperty cell33width = new SimpleDoubleProperty(cells[3][3].layoutXProperty().get());
        
        KeyFrame kf0 = new KeyFrame(Duration.millis(500), new KeyValue(cells[0][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[0][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[0][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[0][3].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cell10height, cell10height.get())
        );

        KeyFrame kf1 = new KeyFrame(Duration.millis(1500), new KeyValue(cell11width, cell11width.get()),
                new KeyValue(cell12width, cell12width.get()),
                new KeyValue(cell13width, cell13width.get()),
                new KeyValue(cell10width, cell10width.get()),
                new KeyValue(cell10height, stateArrayCellsContainer.widthProperty().divide(6).multiply(0).get())
        );

        KeyFrame kf2 = new KeyFrame(Duration.millis(2500), new KeyValue(cell11width, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell12width, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell13width, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get())
        );
      
        KeyFrame kf3 = new KeyFrame(Duration.millis(4500), new KeyValue(cell10height, stateArrayCellsContainer.widthProperty().divide(6).multiply(0).get()),
                new KeyValue(cell10width, stateArrayCellsContainer.widthProperty().divide(6).multiply(4).get())
        );

        KeyFrame kf4 = new KeyFrame(Duration.millis(5500), new KeyValue(cell10height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()));

        KeyFrame kf5 = new KeyFrame(Duration.millis(5500), new KeyValue(cells[1][0].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[1][1].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[1][2].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[1][3].rectangle.fillProperty(), Color.RED),
                new KeyValue(cell10height, cell10height.get())
        );

        KeyFrame kf6 = new KeyFrame(Duration.millis(6000), new KeyValue(cells[1][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[1][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[1][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[1][3].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cell20height, cell20height.get()),
                new KeyValue(cell21height, cell21height.get())
        );
               
        KeyFrame kf7 = new KeyFrame(Duration.millis(7000),
                new KeyValue(cell20height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell21height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell22width, cell22width.get()),
                new KeyValue(cell23width, cell23width.get()),
                new KeyValue(cell21width, cell21width.get()),
                new KeyValue(cell20width, cell20width.get())
        );
         
        KeyFrame kf8 = new KeyFrame(Duration.millis(9000),
                new KeyValue(cell20width, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get()),
                new KeyValue(cell21width, stateArrayCellsContainer.widthProperty().divide(6).multiply(4).get()),
                new KeyValue(cell22width, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell23width, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell20height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell21height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get())
        );
       
        KeyFrame kf9 = new KeyFrame(Duration.millis(10000),
                new KeyValue(cell20height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell21height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cells[2][0].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[2][1].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[2][2].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[2][3].rectangle.fillProperty(), Color.RED)
        );

        KeyFrame kf10 = new KeyFrame(Duration.millis(10500),
                new KeyValue(cells[2][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[2][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[2][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[2][3].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cell33height, cell33height.get())
        );
      
        KeyFrame kf11 = new KeyFrame(Duration.millis(11500),
                new KeyValue(cell33height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell33width, cell33width.get()),
                new KeyValue(cell30width, cell30width.get()),
                new KeyValue(cell31width, cell31width.get()),
                new KeyValue(cell32width, cell32width.get())
        );

        KeyFrame kf12 = new KeyFrame(Duration.millis(12500),
                new KeyValue(cell30width, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell31width, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get()),
                new KeyValue(cell32width, stateArrayCellsContainer.widthProperty().divide(6).multiply(4).get())
        );

        KeyFrame kf13 = new KeyFrame(Duration.millis(14500),
                new KeyValue(cell33height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell33width, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get())
        );

        KeyFrame kf14 = new KeyFrame(Duration.millis(15500),
                new KeyValue(cell33height, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get()),
                new KeyValue(cells[3][0].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[3][1].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[3][2].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[3][3].rectangle.fillProperty(), Color.RED)
        );

        KeyFrame kf15 = new KeyFrame(Duration.millis(16000),
                new KeyValue(cells[3][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[3][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[3][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[3][3].rectangle.fillProperty(), Color.YELLOWGREEN)
        );

        cells[1][0].layoutYProperty().bind(cell10height);
        cells[1][0].toFront();
        cells[1][0].layoutXProperty().bind(cell10width);
        cells[1][1].layoutXProperty().bind(cell11width);
        cells[1][2].layoutXProperty().bind(cell12width);
        cells[1][3].layoutXProperty().bind(cell13width);
        cells[2][0].layoutYProperty().bind(cell20height);
        cells[2][1].layoutYProperty().bind(cell21height);
        cells[2][0].layoutXProperty().bind(cell20width);
        cells[2][1].layoutXProperty().bind(cell21width);
        cells[2][2].layoutXProperty().bind(cell22width);
        cells[2][3].layoutXProperty().bind(cell23width);
        cells[2][0].toFront();
        cells[2][1].toFront();
        cells[3][3].layoutYProperty().bind(cell33height);
        cells[3][0].layoutXProperty().bind(cell30width);
        cells[3][1].layoutXProperty().bind(cell31width);
        cells[3][2].layoutXProperty().bind(cell32width);
        cells[3][3].layoutXProperty().bind(cell33width);
        cells[3][3].toFront();
        anim.getKeyFrames().addAll(kf0, kf1, kf2, kf3, kf4, kf5, kf6, kf7, kf8, kf9, kf10, kf11, kf12, kf13, kf14, kf15);
        
        return anim;
    }

    Timeline invereSwiftRowsAnim() {
        refresh();
        Timeline anim = new Timeline();
        SimpleDoubleProperty cell13height = new SimpleDoubleProperty(cells[1][3].layoutYProperty().get());
        SimpleDoubleProperty cell10width = new SimpleDoubleProperty(cells[1][0].layoutXProperty().get());
        SimpleDoubleProperty cell11width = new SimpleDoubleProperty(cells[1][1].layoutXProperty().get());
        SimpleDoubleProperty cell12width = new SimpleDoubleProperty(cells[1][2].layoutXProperty().get());
        SimpleDoubleProperty cell13width = new SimpleDoubleProperty(cells[1][3].layoutXProperty().get());
        SimpleDoubleProperty cell20width = new SimpleDoubleProperty(cells[2][0].layoutXProperty().get());
        SimpleDoubleProperty cell21width = new SimpleDoubleProperty(cells[2][1].layoutXProperty().get());
        SimpleDoubleProperty cell22width = new SimpleDoubleProperty(cells[2][2].layoutXProperty().get());
        SimpleDoubleProperty cell23width = new SimpleDoubleProperty(cells[2][3].layoutXProperty().get());
        SimpleDoubleProperty cell22height = new SimpleDoubleProperty(cells[2][2].layoutYProperty().get());
        SimpleDoubleProperty cell23height = new SimpleDoubleProperty(cells[2][3].layoutYProperty().get());
        SimpleDoubleProperty cell30height = new SimpleDoubleProperty(cells[3][0].layoutYProperty().get());
        SimpleDoubleProperty cell30width = new SimpleDoubleProperty(cells[3][0].layoutXProperty().get());
        SimpleDoubleProperty cell31width = new SimpleDoubleProperty(cells[3][1].layoutXProperty().get());
        SimpleDoubleProperty cell32width = new SimpleDoubleProperty(cells[3][2].layoutXProperty().get());
        SimpleDoubleProperty cell33width = new SimpleDoubleProperty(cells[3][3].layoutXProperty().get());

        KeyFrame kf0 = new KeyFrame(Duration.millis(500), new KeyValue(cells[0][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[0][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[0][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[0][3].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cell13height, cell13height.get())
        );

        KeyFrame kf1 = new KeyFrame(Duration.millis(1500), new KeyValue(cell11width, cell11width.get()),
                new KeyValue(cell12width, cell12width.get()),
                new KeyValue(cell13width, cell13width.get()),
                new KeyValue(cell10width, cell10width.get()),
                new KeyValue(cell13height, stateArrayCellsContainer.widthProperty().divide(6).multiply(0).get())
        );

        KeyFrame kf2 = new KeyFrame(Duration.millis(2500), new KeyValue(cell10width, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell11width, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get()),
                new KeyValue(cell12width, stateArrayCellsContainer.widthProperty().divide(6).multiply(4).get())
        );

        KeyFrame kf3 = new KeyFrame(Duration.millis(4500), new KeyValue(cell13height, stateArrayCellsContainer.widthProperty().divide(6).multiply(0).get()),
                new KeyValue(cell13width, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get())
        );

        KeyFrame kf4 = new KeyFrame(Duration.millis(5500), new KeyValue(cell13height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()));

        KeyFrame kf5 = new KeyFrame(Duration.millis(5500), new KeyValue(cells[1][0].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[1][1].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[1][2].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[1][3].rectangle.fillProperty(), Color.RED),
                new KeyValue(cell13height, cell13height.get())
        );

        KeyFrame kf6 = new KeyFrame(Duration.millis(6000), new KeyValue(cells[1][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[1][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[1][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[1][3].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cell22height, cell22height.get()),
                new KeyValue(cell23height, cell23height.get())
        );

        KeyFrame kf7 = new KeyFrame(Duration.millis(7000),
                new KeyValue(cell22height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell23height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell22width, cell22width.get()),
                new KeyValue(cell23width, cell23width.get()),
                new KeyValue(cell21width, cell21width.get()),
                new KeyValue(cell20width, cell20width.get())
        );

        KeyFrame kf8 = new KeyFrame(Duration.millis(9000),
                new KeyValue(cell20width, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get()),
                new KeyValue(cell21width, stateArrayCellsContainer.widthProperty().divide(6).multiply(4).get()),
                new KeyValue(cell22width, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell23width, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell22height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell23height, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get())
        );

        KeyFrame kf9 = new KeyFrame(Duration.millis(10000),
                new KeyValue(cell22height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell23height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cells[2][0].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[2][1].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[2][2].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[2][3].rectangle.fillProperty(), Color.RED)
        );

        KeyFrame kf10 = new KeyFrame(Duration.millis(10500),
                new KeyValue(cells[2][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[2][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[2][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[2][3].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cell30height, cell30height.get())
        );

        KeyFrame kf11 = new KeyFrame(Duration.millis(11500),
                new KeyValue(cell30height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell33width, cell33width.get()),
                new KeyValue(cell30width, cell30width.get()),
                new KeyValue(cell31width, cell31width.get()),
                new KeyValue(cell32width, cell32width.get())
        );

        KeyFrame kf12 = new KeyFrame(Duration.millis(12500),
                new KeyValue(cell31width, stateArrayCellsContainer.widthProperty().divide(6).multiply(1).get()),
                new KeyValue(cell32width, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell33width, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get())
        );

        KeyFrame kf13 = new KeyFrame(Duration.millis(14500),
                new KeyValue(cell30height, stateArrayCellsContainer.widthProperty().divide(6).multiply(2).get()),
                new KeyValue(cell30width, stateArrayCellsContainer.widthProperty().divide(6).multiply(4).get())
        );

        KeyFrame kf14 = new KeyFrame(Duration.millis(15500),
                new KeyValue(cell30height, stateArrayCellsContainer.widthProperty().divide(6).multiply(3).get()),
                new KeyValue(cells[3][0].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[3][1].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[3][2].rectangle.fillProperty(), Color.RED),
                new KeyValue(cells[3][3].rectangle.fillProperty(), Color.RED)
        );

        KeyFrame kf15 = new KeyFrame(Duration.millis(16000),
                new KeyValue(cells[3][0].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[3][1].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[3][2].rectangle.fillProperty(), Color.YELLOWGREEN),
                new KeyValue(cells[3][3].rectangle.fillProperty(), Color.YELLOWGREEN)
        );

        cells[1][3].layoutYProperty().bind(cell13height);
        cells[1][3].toFront();
        cells[1][0].layoutXProperty().bind(cell10width);
        cells[1][1].layoutXProperty().bind(cell11width);
        cells[1][2].layoutXProperty().bind(cell12width);
        cells[1][3].layoutXProperty().bind(cell13width);
        cells[2][2].layoutYProperty().bind(cell22height);
        cells[2][3].layoutYProperty().bind(cell23height);
        cells[2][0].layoutXProperty().bind(cell20width);
        cells[2][1].layoutXProperty().bind(cell21width);
        cells[2][2].layoutXProperty().bind(cell22width);
        cells[2][3].layoutXProperty().bind(cell23width);
        cells[2][2].toFront();
        cells[2][3].toFront();
        cells[3][0].layoutYProperty().bind(cell30height);
        cells[3][0].layoutXProperty().bind(cell30width);
        cells[3][1].layoutXProperty().bind(cell31width);
        cells[3][2].layoutXProperty().bind(cell32width);
        cells[3][3].layoutXProperty().bind(cell33width);
        cells[3][0].toFront();
        anim.getKeyFrames().addAll(kf0, kf1, kf2, kf3, kf4, kf5, kf6, kf7, kf8, kf9, kf10, kf11, kf12, kf13, kf14, kf15);
        return anim;
    }

    Timeline substiteBytesAnimation() {
        refresh();
        Timeline anim = new Timeline();
        int timeInMillis = 0;
        SimpleBooleanProperty textVisible = new SimpleBooleanProperty(false);
        Text textForAnimation = new Text();
        textForAnimation.setFont(new Font(16));
        Text textForAnimation0 = new Text();
        textForAnimation.setFont(Font.font("system", FontWeight.BOLD, 16));
        textForAnimation0.setFont(new Font(16));
        animationText.visibleProperty().bind(textVisible);
        animationText.getChildren().addAll(textForAnimation0, textForAnimation);
        animationText.layout();
        textForAnimation.setFill(Color.RED);
        SimpleStringProperty textStringProperty = new SimpleStringProperty();
        SimpleStringProperty firstStringProperty = new SimpleStringProperty();
        textForAnimation.textProperty().bind(textStringProperty);
        textForAnimation0.textProperty().bind(firstStringProperty);
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                StateArrayCell cell = cells[j][i];

                KeyFrame kf0 = new KeyFrame(Duration.millis(timeInMillis), new KeyValue(
                        cell.rectangle.fillProperty(), Color.RED),
                        new KeyValue(textVisible, false)
                );
                timeInMillis += 500;
                KeyFrame kf1 = new KeyFrame(Duration.millis(timeInMillis), new KeyValue(
                        cell.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                        new KeyValue(textVisible, false)
                );
                timeInMillis += 500;
                KeyFrame kf2 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(textVisible, true),
                        new KeyValue(firstStringProperty, "Byte: "),
                        new KeyValue(textStringProperty, cell.text.getText())
                );
                timeInMillis += 1500;
                KeyFrame kf3 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(firstStringProperty, "First half: row index: "),
                        new KeyValue(textStringProperty, String.valueOf(cell.text.getText().charAt(0)) + " (" + Character.digit(cell.text.getText().charAt(0), 16) + ")")
                );
                timeInMillis += 1500;
                KeyFrame kf4 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(firstStringProperty, "Second half: column index: "),
                        new KeyValue(textStringProperty, String.valueOf(cell.text.getText().charAt(1)) + " (" + Character.digit(cell.text.getText().charAt(1), 16) + ")")
                );

                timeInMillis += 1500;
                KeyFrame kf5;
                if (encryptionMode) {
                    kf5 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(firstStringProperty, "New Byte is SBox cell  "),
                            new KeyValue(textStringProperty, String.valueOf("[" + Character.digit(cell.text.getText().charAt(0), 16)) + ":" + Character.digit(cell.text.getText().charAt(1), 16) + "]"),
                            new KeyValue(textVisible, true)
                    );
                } else {
                    kf5 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(firstStringProperty, "New Byte is Inverse SBox cell  "),
                            new KeyValue(textStringProperty, String.valueOf("[" + Character.digit(cell.text.getText().charAt(0), 16)) + ":" + Character.digit(cell.text.getText().charAt(1), 16) + "]"),
                            new KeyValue(textVisible, true)
                    );
                }
                timeInMillis += 500;
                
                anim = sbox.addHighlightingFrames(anim, Integer.valueOf(Character.digit(cell.text.getText().charAt(0), 16)), Integer.valueOf(Character.digit(cell.text.getText().charAt(1), 16)), timeInMillis);
               
                timeInMillis += 4000;
                KeyFrame kf6 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(cell.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                        new KeyValue(textVisible, false),
                        new KeyValue(cell.text.textProperty(), cell.text.textProperty().get()),
                        new KeyValue(cell.text.textProperty(), sbox.getTextOfCell(Integer.valueOf(Character.digit(cell.text.getText().charAt(0), 16)), Integer.valueOf(Character.digit(cell.text.getText().charAt(1), 16))))
                );

                timeInMillis += 1000;
                KeyFrame kf7 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(cell.rectangle.fillProperty(), Color.YELLOWGREEN),
                        new KeyValue(textVisible, false)
                );

                anim.getKeyFrames().addAll(kf0, kf1, kf2, kf3, kf4, kf5, kf6, kf7);
            }
        }
        return anim;
    }

    Timeline addKeyAnimation() {
        refresh();
        Timeline anim = new Timeline();
        int timeInMillis = 0;
        Font boldFont = Font.font("system", FontWeight.BOLD, 16);
        SimpleBooleanProperty textVisible = new SimpleBooleanProperty(false);
        Text textForAnimationRed = new Text();
        textForAnimationRed.setFont(boldFont);
        Text textForAnimation1 = new Text();
        textForAnimation1.setFont(new Font(16));
        Text textForAnimation2 = new Text();
        textForAnimation2.setFont(new Font(16));
        Text textForAnimationBlue = new Text();
        textForAnimationBlue.setFont(boldFont);
        Text textForAnimation4 = new Text();
        textForAnimation4.setFont(new Font(16));
        Text textForAnimationGreen = new Text();
        textForAnimationGreen.setFont(boldFont);
        textForAnimationRed.setFill(Color.RED);
        textForAnimationBlue.setFill(Color.CORNFLOWERBLUE);
        textForAnimationGreen.setFill(Color.YELLOWGREEN);
        animationText.getChildren().addAll(textForAnimation1, textForAnimationRed, textForAnimation2, textForAnimationBlue, textForAnimation4, textForAnimationGreen);
        animationText.visibleProperty().bind(textVisible);
        SimpleStringProperty redStringProperty = new SimpleStringProperty();
        SimpleStringProperty firstStringProperty = new SimpleStringProperty();
        SimpleStringProperty secondStringProperty = new SimpleStringProperty();
        SimpleStringProperty blueStringProperty = new SimpleStringProperty();
        SimpleStringProperty thirdStringProperty = new SimpleStringProperty();
        SimpleStringProperty greenStringProperty = new SimpleStringProperty();
        textForAnimation1.textProperty().bind(firstStringProperty);
        textForAnimationRed.textProperty().bind(redStringProperty);
        textForAnimation2.textProperty().bind(secondStringProperty);
        textForAnimationBlue.textProperty().bind(blueStringProperty);
        textForAnimation4.textProperty().bind(thirdStringProperty);
        textForAnimationGreen.textProperty().bind(greenStringProperty);
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                StateArrayCell cell = cells[j][i];

                KeyFrame kf0 = new KeyFrame(Duration.millis(timeInMillis), new KeyValue(
                        cell.rectangle.fillProperty(), Color.RED),
                        new KeyValue(textVisible, false)
                );
                timeInMillis += 500;
                KeyFrame kf1 = new KeyFrame(Duration.millis(timeInMillis), new KeyValue(
                        cell.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                        new KeyValue(textVisible, false)
                );
                timeInMillis += 500;
                KeyFrame kf2 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(textVisible, true),
                        new KeyValue(firstStringProperty, "Byte: "),
                        new KeyValue(redStringProperty, cell.text.getText()),
                        new KeyValue(secondStringProperty, " is XORed with key byte ")
                );

                timeInMillis += 2000;
                KeyFrame kf3;
                if (encryptionMode) {

                    kf3 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(firstStringProperty, "Corresponding key byte"),
                            new KeyValue(redStringProperty, " "),
                            new KeyValue(secondStringProperty, "is: "),
                            new KeyValue(blueStringProperty, keyField.getTextOfCell((phaseIndex + 1) / 4, i * 4 + j))
                    );
                } else {

                    kf3 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(firstStringProperty, "Corresponding key byte"),
                            new KeyValue(redStringProperty, " "),
                            new KeyValue(secondStringProperty, "is: "),
                            new KeyValue(blueStringProperty, keyField.getTextOfCell(10 - ((phaseIndex + 1) / 4), i * 4 + j))
                    );

                }
                if (encryptionMode) {
                    anim = keyField.addHighlightingFrames(anim, (phaseIndex + 1) / 4, i * 4 + j, timeInMillis);
                } else {
                    anim = keyField.addHighlightingFrames(anim, (10 - (phaseIndex + 1) / 4), i * 4 + j, timeInMillis);
                }
                timeInMillis += 2000;
                KeyFrame kf4;
                if (encryptionMode) {
                    kf4 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(firstStringProperty, "New byte is: "),
                            new KeyValue(redStringProperty, cell.text.getText()),
                            new KeyValue(secondStringProperty, " \u2295 "),
                            new KeyValue(thirdStringProperty, " = "),
                            new KeyValue(greenStringProperty, String.format("%02X", ((Integer.parseInt(cell.text.getText(), 16) & 0xff) ^ Integer.parseInt(keyField.getTextOfCell(((phaseIndex + 1) / 4), i * 4 + j), 16) & 0xff)))
                    );
                } else {
                    kf4 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(firstStringProperty, "New byte is: "),
                            new KeyValue(redStringProperty, cell.text.getText()),
                            new KeyValue(secondStringProperty, " \u2295 "),
                            new KeyValue(thirdStringProperty, " = "),
                            new KeyValue(greenStringProperty, String.format("%02X", ((Integer.parseInt(cell.text.getText(), 16) & 0xff) ^ Integer.parseInt(keyField.getTextOfCell((10 - ((phaseIndex + 1) / 4)), i * 4 + j), 16) & 0xff)))
                    );
                }
                timeInMillis += 3000;
                KeyFrame kf6;
                if (encryptionMode) {
                    kf6 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(cell.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                            new KeyValue(cell.text.textProperty(), cell.text.textProperty().get()),
                            new KeyValue(cell.text.textProperty(), String.format("%02X", ((Integer.parseInt(cell.text.getText(), 16) & 0xff) ^ Integer.parseInt(keyField.getTextOfCell(((phaseIndex + 1) / 4), i * 4 + j), 16) & 0xff)))
                    );
                } else {
                    kf6 = new KeyFrame(Duration.millis(timeInMillis),
                            new KeyValue(cell.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                            new KeyValue(cell.text.textProperty(), cell.text.textProperty().get()),
                            new KeyValue(cell.text.textProperty(), String.format("%02X", ((Integer.parseInt(cell.text.getText(), 16) & 0xff) ^ Integer.parseInt(keyField.getTextOfCell((10 - ((phaseIndex + 1) / 4)), i * 4 + j), 16) & 0xff)))
                    );
                }
                timeInMillis += 500;
                KeyFrame kf7 = new KeyFrame(Duration.millis(timeInMillis),
                        new KeyValue(cell.rectangle.fillProperty(), Color.YELLOWGREEN),
                        new KeyValue(secondStringProperty, ""),
                        new KeyValue(thirdStringProperty, ""),
                        new KeyValue(greenStringProperty, ""),
                        new KeyValue(blueStringProperty, ""),
                        new KeyValue(textVisible, false)
                );

                anim.getKeyFrames().addAll(kf0, kf1, kf2, kf3, kf4, kf6, kf7);
            }
        }
        return anim;
    }

    Timeline mixColumnsAnimation() {
        refresh();
        byte[][] mixColumnsOutput = data.getPhase(blockIndex, phaseIndex + 1);
        Timeline anim = new Timeline();
        int timeInMillis = 0;
        Font boldFont = Font.font("system", FontWeight.BOLD, 16);
        SimpleBooleanProperty textVisible = new SimpleBooleanProperty(false);
        Text textForAnimation1 = new Text();
        textForAnimation1.setFont(new Font(16));
        Text textForAnimationBlue = new Text();
        textForAnimationBlue.setFont(boldFont);
        Text textForAnimationGreen = new Text();
        textForAnimationGreen.setFont(boldFont);
        Text textForAnimation2 = new Text();
        textForAnimation2.setFont(new Font(16));
        Text textForAnimationOrange1 = new Text();
        textForAnimationOrange1.setFont(boldFont);
        Text textForAnimation3 = new Text();
        textForAnimation3.setFont(new Font(16));
        Text textForAnimationOrange2 = new Text();
        textForAnimationOrange2.setFont(boldFont);
        Text textForAnimation4 = new Text();
        textForAnimation4.setFont(new Font(16));
        Text textForAnimationOrange3 = new Text();
        textForAnimationOrange3.setFont(boldFont);
        Text textForAnimation5 = new Text();
        textForAnimation5.setFont(new Font(16));
        Text textForAnimationOrange4 = new Text();
        textForAnimationOrange4.setFont(boldFont);
        Text textForAnimation6 = new Text();
        textForAnimation6.setFont(new Font(16));
        Text textForAnimationGreen2 = new Text();
        textForAnimationGreen2.setFont(boldFont);
        textForAnimationOrange1.setFill(Color.DARKORANGE);
        textForAnimationOrange2.setFill(Color.DARKORANGE);
        textForAnimationOrange3.setFill(Color.DARKORANGE);
        textForAnimationOrange4.setFill(Color.DARKORANGE);
        textForAnimationBlue.setFill(Color.CORNFLOWERBLUE);
        textForAnimationGreen.setFill(Color.YELLOWGREEN);
        textForAnimationGreen2.setFill(Color.YELLOWGREEN);
        
        animationText.getChildren().addAll(textForAnimation1, textForAnimationBlue, textForAnimationGreen, textForAnimation2, textForAnimationOrange1, textForAnimation3,
                textForAnimationOrange2, textForAnimation4, textForAnimationOrange3, textForAnimation5, textForAnimationOrange4, textForAnimation6, textForAnimationGreen2
        );
        animationText.visibleProperty().bind(textVisible);
        
        SimpleStringProperty orange1StringProperty = new SimpleStringProperty();
        SimpleStringProperty orange2StringProperty = new SimpleStringProperty();
        SimpleStringProperty orange3StringProperty = new SimpleStringProperty();
        SimpleStringProperty orange4StringProperty = new SimpleStringProperty();
        SimpleStringProperty firstStringProperty = new SimpleStringProperty();
        SimpleStringProperty secondStringProperty = new SimpleStringProperty();
        SimpleStringProperty blueStringProperty = new SimpleStringProperty();
        SimpleStringProperty thirdStringProperty = new SimpleStringProperty();
        SimpleStringProperty fourthStringProperty = new SimpleStringProperty();
        SimpleStringProperty fifthStringProperty = new SimpleStringProperty();
        SimpleStringProperty sixthStringProperty = new SimpleStringProperty();
        SimpleStringProperty greenStringProperty = new SimpleStringProperty();
        SimpleStringProperty secondGreenStringProperty = new SimpleStringProperty();
        
        textForAnimation1.textProperty().bind(firstStringProperty);
        textForAnimation2.textProperty().bind(secondStringProperty);
        textForAnimation3.textProperty().bind(thirdStringProperty);
        textForAnimation4.textProperty().bind(fourthStringProperty);
        textForAnimation5.textProperty().bind(fifthStringProperty);
        textForAnimation6.textProperty().bind(sixthStringProperty);
        textForAnimationOrange1.textProperty().bind(orange1StringProperty);
        textForAnimationOrange2.textProperty().bind(orange2StringProperty);
        textForAnimationOrange3.textProperty().bind(orange3StringProperty);
        textForAnimationOrange4.textProperty().bind(orange4StringProperty);
        textForAnimationBlue.textProperty().bind(blueStringProperty);
        textForAnimationGreen.textProperty().bind(greenStringProperty);
        textForAnimationGreen2.textProperty().bind(secondGreenStringProperty);
        
        for (int i = 0; i < 4; i++) {
            StateArrayCell cell0 = cells[0][i];
            StateArrayCell cell1 = cells[1][i];
            StateArrayCell cell2 = cells[2][i];
            StateArrayCell cell3 = cells[3][i];

            KeyFrame kf0 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.RED),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.RED),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.RED),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.RED),
                    new KeyValue(textVisible, false)
            );
            timeInMillis += 500;
            KeyFrame kf1 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(textVisible, false)
            );
            timeInMillis += 500;
            KeyFrame kf2 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "First byte of column "),
                    new KeyValue(blueStringProperty, cell0.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: ")
            );

            timeInMillis += 1000;
            KeyFrame kf3;

            kf3 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(2 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (3 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, " = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[0][i]))
            );

            KeyFrame kf4;
            timeInMillis += 2000;

            kf4 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(2 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (3 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, " = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[0][i])),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.CORNFLOWERBLUE)
            );

            KeyFrame kf5;
            timeInMillis += 500;

            kf5 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(2 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 ("),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, " = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[0][i]))
            );

            KeyFrame kf6;
            timeInMillis += 500;

            kf6 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "Second byte of column "),
                    new KeyValue(blueStringProperty, cell1.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: "),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            timeInMillis += 1000;
            KeyFrame kf7 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, " "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, " \u2295 (2 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (3 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, " = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[1][i]))
            );

            timeInMillis += 2000;
            KeyFrame kf8;
            kf8 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, " "),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, " \u2295 (2 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (3 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, " = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[1][i]))
            );

            KeyFrame kf9;
            timeInMillis += 500;

            kf9 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(2 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 ("),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, " = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[1][i]))
            );

            KeyFrame kf10;
            timeInMillis += 500;
            kf10 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "Third byte of column "),
                    new KeyValue(blueStringProperty, cell2.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: "),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            timeInMillis += 1000;
            KeyFrame kf11 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, " "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, " \u2295 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, " \u2295 (2 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (3 \u2219"),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[2][i]))
            );

            timeInMillis += 2000;
            KeyFrame kf12;
            kf12 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, " "),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, " \u2295 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, " \u2295 (2 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (3 \u2219"),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[2][i]))
            );

            timeInMillis += 500;

            KeyFrame kf13 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(secondStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, " \u2295 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, " \u2295 (2 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (3 \u2219"),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[2][i]))
            );

            KeyFrame kf14;
            timeInMillis += 500;
            kf14 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "Fourth byte of column "),
                    new KeyValue(blueStringProperty, cell3.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: "),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            timeInMillis += 1000;
            KeyFrame kf15 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(3 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, " \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 (2 \u2219"),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[3][i]))
            );

            timeInMillis += 2000;
            KeyFrame kf16;
            kf16 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(3 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, " \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 (2 \u2219"),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[3][i]))
            );

            timeInMillis += 500;

            KeyFrame kf17 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(secondStringProperty, "(3 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, " \u2295 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, " \u2295 (2 \u2219"),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[3][i]))
            );

            KeyFrame kf18;
            timeInMillis += 500;
            kf18 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, "New bytes of altered column are:"),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, String.format("%02X", mixColumnsOutput[0][i]) + " " + String.format("%02X", mixColumnsOutput[1][i]) + " " + String.format("%02X", mixColumnsOutput[2][i]) + " " + String.format("%02X", mixColumnsOutput[3][i])),
                    new KeyValue(secondStringProperty, ""),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            KeyFrame kf19;
            timeInMillis += 2000;
            kf19 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, "New bytes of altered column are:"),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, String.format("%02X", mixColumnsOutput[0][i]) + " " + String.format("%02X", mixColumnsOutput[1][i]) + " " + String.format("%02X", mixColumnsOutput[2][i]) + " " + String.format("%02X", mixColumnsOutput[3][i])),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell0.text.textProperty(), cell0.text.textProperty().get()),
                    new KeyValue(cell1.text.textProperty(), cell1.text.textProperty().get()),
                    new KeyValue(cell2.text.textProperty(), cell2.text.textProperty().get()),
                    new KeyValue(cell3.text.textProperty(), cell3.text.textProperty().get())
            );

            KeyFrame kf20;
            timeInMillis += 500;
            kf20 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, "New bytes of altered column are:"),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, String.format("%02X", mixColumnsOutput[0][i]) + " " + String.format("%02X", mixColumnsOutput[1][i]) + " " + String.format("%02X", mixColumnsOutput[2][i]) + " " + String.format("%02X", mixColumnsOutput[3][i])),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell0.text.textProperty(), String.format("%02X", mixColumnsOutput[0][i])),
                    new KeyValue(cell1.text.textProperty(), String.format("%02X", mixColumnsOutput[1][i])),
                    new KeyValue(cell2.text.textProperty(), String.format("%02X", mixColumnsOutput[2][i])),
                    new KeyValue(cell3.text.textProperty(), String.format("%02X", mixColumnsOutput[3][i]))
            );

            KeyFrame kf21;
            timeInMillis += 500;
            kf21 = new KeyFrame(Duration.millis(timeInMillis), new KeyValue(textVisible, false));
            anim.getKeyFrames().addAll(kf0, kf1, kf2, kf3, kf4, kf5, kf6, kf7, kf8, kf9, kf10, kf11, kf12, kf13, kf14, kf15, kf16, kf17, kf18, kf19, kf20, kf21);
        }
        return anim;
    }

    Timeline inverseMixColumnsAnimation() {
        refresh();
        byte[][] mixColumnsOutput = data.getPhase(blockIndex, phaseIndex + 1);
        Timeline anim = new Timeline();
        int timeInMillis = 0;
        Font boldFont = Font.font("system", FontWeight.BOLD, 16);
        SimpleBooleanProperty textVisible = new SimpleBooleanProperty(false);
        
        Text textForAnimation1 = new Text();
        textForAnimation1.setFont(new Font(16));
        Text textForAnimationBlue = new Text();
        textForAnimationBlue.setFont(boldFont);
        Text textForAnimationGreen = new Text();
        textForAnimationGreen.setFont(boldFont);
        Text textForAnimation2 = new Text();
        textForAnimation2.setFont(new Font(16));
        Text textForAnimationOrange1 = new Text();
        textForAnimationOrange1.setFont(boldFont);
        Text textForAnimation3 = new Text();
        textForAnimation3.setFont(new Font(16));
        Text textForAnimationOrange2 = new Text();
        textForAnimationOrange2.setFont(boldFont);
        Text textForAnimation4 = new Text();
        textForAnimation4.setFont(new Font(16));
        Text textForAnimationOrange3 = new Text();
        textForAnimationOrange3.setFont(boldFont);
        Text textForAnimation5 = new Text();
        textForAnimation5.setFont(new Font(16));
        Text textForAnimationOrange4 = new Text();
        textForAnimationOrange4.setFont(boldFont);
        Text textForAnimation6 = new Text();
        textForAnimation6.setFont(new Font(16));
        Text textForAnimationGreen2 = new Text();
        
        textForAnimationGreen2.setFont(boldFont);
        textForAnimationOrange1.setFill(Color.DARKORANGE);
        textForAnimationOrange2.setFill(Color.DARKORANGE);
        textForAnimationOrange3.setFill(Color.DARKORANGE);
        textForAnimationOrange4.setFill(Color.DARKORANGE);
        textForAnimationBlue.setFill(Color.CORNFLOWERBLUE);
        textForAnimationGreen.setFill(Color.YELLOWGREEN);
        textForAnimationGreen2.setFill(Color.YELLOWGREEN);

        animationText.getChildren().addAll(textForAnimation1, textForAnimationBlue, textForAnimationGreen, textForAnimation2, textForAnimationOrange1, textForAnimation3,
                textForAnimationOrange2, textForAnimation4, textForAnimationOrange3, textForAnimation5, textForAnimationOrange4, textForAnimation6, textForAnimationGreen2
        );
        animationText.visibleProperty().bind(textVisible);
        
        SimpleStringProperty orange1StringProperty = new SimpleStringProperty();
        SimpleStringProperty orange2StringProperty = new SimpleStringProperty();
        SimpleStringProperty orange3StringProperty = new SimpleStringProperty();
        SimpleStringProperty orange4StringProperty = new SimpleStringProperty();
        SimpleStringProperty firstStringProperty = new SimpleStringProperty();
        SimpleStringProperty secondStringProperty = new SimpleStringProperty();
        SimpleStringProperty blueStringProperty = new SimpleStringProperty();
        SimpleStringProperty thirdStringProperty = new SimpleStringProperty();
        SimpleStringProperty fourthStringProperty = new SimpleStringProperty();
        SimpleStringProperty fifthStringProperty = new SimpleStringProperty();
        SimpleStringProperty sixthStringProperty = new SimpleStringProperty();
        SimpleStringProperty greenStringProperty = new SimpleStringProperty();
        SimpleStringProperty secondGreenStringProperty = new SimpleStringProperty();
       
        textForAnimation1.textProperty().bind(firstStringProperty);
        textForAnimation2.textProperty().bind(secondStringProperty);
        textForAnimation3.textProperty().bind(thirdStringProperty);
        textForAnimation4.textProperty().bind(fourthStringProperty);
        textForAnimation5.textProperty().bind(fifthStringProperty);
        textForAnimation6.textProperty().bind(sixthStringProperty);
        textForAnimationOrange1.textProperty().bind(orange1StringProperty);
        textForAnimationOrange2.textProperty().bind(orange2StringProperty);
        textForAnimationOrange3.textProperty().bind(orange3StringProperty);
        textForAnimationOrange4.textProperty().bind(orange4StringProperty);
        textForAnimationBlue.textProperty().bind(blueStringProperty);
        textForAnimationGreen.textProperty().bind(greenStringProperty);
        
        textForAnimationGreen2.textProperty().bind(secondGreenStringProperty);
        for (int i = 0; i < 4; i++) {
            StateArrayCell cell0 = cells[0][i];
            StateArrayCell cell1 = cells[1][i];
            StateArrayCell cell2 = cells[2][i];
            StateArrayCell cell3 = cells[3][i];

            KeyFrame kf0 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.RED),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.RED),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.RED),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.RED),
                    new KeyValue(textVisible, false)
            );
            timeInMillis += 500;
            KeyFrame kf1 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(textVisible, false)
            );
            timeInMillis += 500;
            KeyFrame kf2 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "First byte of column "),
                    new KeyValue(blueStringProperty, cell0.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: ")
            );

            timeInMillis += 1000;
            KeyFrame kf3;

            kf3 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0E \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[0][i]))
            );

            KeyFrame kf4;
            timeInMillis += 2000;

            kf4 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0E \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[0][i])),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.CORNFLOWERBLUE)
            );

            KeyFrame kf5;
            timeInMillis += 500;

            kf5 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0E \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[0][i]))
            );

            KeyFrame kf6;
            timeInMillis += 500;

            kf6 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "Second byte of column "),
                    new KeyValue(blueStringProperty, cell1.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: "),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            timeInMillis += 1000;
            KeyFrame kf7 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(09 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[1][i]))
            );

            timeInMillis += 2000;
            KeyFrame kf8;
            kf8 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(secondStringProperty, "(09 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[1][i]))
            );

            KeyFrame kf9;
            timeInMillis += 500;

            kf9 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(09 \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[1][i]))
            );

            KeyFrame kf10;
            timeInMillis += 500;
            kf10 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "Third byte of column "),
                    new KeyValue(blueStringProperty, cell2.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: "),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            timeInMillis += 1000;
            KeyFrame kf11 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0D \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[2][i]))
            );

            timeInMillis += 2000;
            KeyFrame kf12;
            kf12 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(secondStringProperty, "(0D \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[2][i]))
            );

            timeInMillis += 500;

            KeyFrame kf13 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0D \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0B \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[2][i]))
            );

            KeyFrame kf14;
            timeInMillis += 500;
            kf14 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(firstStringProperty, "Fourth byte of column "),
                    new KeyValue(blueStringProperty, cell3.text.getText()),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "is recounted as: "),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            timeInMillis += 1000;
            KeyFrame kf15 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0B \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[3][i]))
            );

            timeInMillis += 2000;
            KeyFrame kf16;
            kf16 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, " "),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.CORNFLOWERBLUE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0B \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[3][i]))
            );

            timeInMillis += 500;

            KeyFrame kf17 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(firstStringProperty, ""),
                    new KeyValue(textVisible, false),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(secondStringProperty, "(0B \u2219 "),
                    new KeyValue(orange1StringProperty, cell0.text.getText()),
                    new KeyValue(thirdStringProperty, ") \u2295 (0D \u2219 "),
                    new KeyValue(orange2StringProperty, cell1.text.getText()),
                    new KeyValue(fourthStringProperty, ") \u2295 (09 \u2219 "),
                    new KeyValue(orange3StringProperty, cell2.text.getText()),
                    new KeyValue(fifthStringProperty, ") \u2295 (0E \u2219 "),
                    new KeyValue(orange4StringProperty, cell3.text.getText()),
                    new KeyValue(sixthStringProperty, ") = "),
                    new KeyValue(secondGreenStringProperty, String.format("%02X", mixColumnsOutput[3][i]))
            );

            KeyFrame kf18;
            timeInMillis += 500;
            kf18 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, "New bytes of altered column are:"),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, String.format("%02X", mixColumnsOutput[0][i]) + " " + String.format("%02X", mixColumnsOutput[1][i]) + " " + String.format("%02X", mixColumnsOutput[2][i]) + " " + String.format("%02X", mixColumnsOutput[3][i])),
                    new KeyValue(secondStringProperty, ""),
                    new KeyValue(orange1StringProperty, ""),
                    new KeyValue(thirdStringProperty, ""),
                    new KeyValue(orange2StringProperty, ""),
                    new KeyValue(fourthStringProperty, ""),
                    new KeyValue(orange3StringProperty, ""),
                    new KeyValue(fifthStringProperty, ""),
                    new KeyValue(orange4StringProperty, ""),
                    new KeyValue(sixthStringProperty, ""),
                    new KeyValue(secondGreenStringProperty, "")
            );

            KeyFrame kf19;
            timeInMillis += 2000;
            kf19 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, "New bytes of altered column are:"),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, String.format("%02X", mixColumnsOutput[0][i]) + " " + String.format("%02X", mixColumnsOutput[1][i]) + " " + String.format("%02X", mixColumnsOutput[2][i]) + " " + String.format("%02X", mixColumnsOutput[3][i])),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.DARKORANGE),
                    new KeyValue(cell0.text.textProperty(), cell0.text.textProperty().get()),
                    new KeyValue(cell1.text.textProperty(), cell1.text.textProperty().get()),
                    new KeyValue(cell2.text.textProperty(), cell2.text.textProperty().get()),
                    new KeyValue(cell3.text.textProperty(), cell3.text.textProperty().get())
            );

            KeyFrame kf20;
            timeInMillis += 500;
            kf20 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, true),
                    new KeyValue(firstStringProperty, "New bytes of altered column are:"),
                    new KeyValue(blueStringProperty, " "),
                    new KeyValue(greenStringProperty, String.format("%02X", mixColumnsOutput[0][i]) + " " + String.format("%02X", mixColumnsOutput[1][i]) + " " + String.format("%02X", mixColumnsOutput[2][i]) + " " + String.format("%02X", mixColumnsOutput[3][i])),
                    new KeyValue(cell0.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell1.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell2.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell3.rectangle.fillProperty(), Color.YELLOWGREEN),
                    new KeyValue(cell0.text.textProperty(), String.format("%02X", mixColumnsOutput[0][i])),
                    new KeyValue(cell1.text.textProperty(), String.format("%02X", mixColumnsOutput[1][i])),
                    new KeyValue(cell2.text.textProperty(), String.format("%02X", mixColumnsOutput[2][i])),
                    new KeyValue(cell3.text.textProperty(), String.format("%02X", mixColumnsOutput[3][i]))
            );

            KeyFrame kf21;
            timeInMillis += 500;
            kf21 = new KeyFrame(Duration.millis(timeInMillis),
                    new KeyValue(textVisible, false)
            );
            anim.getKeyFrames().addAll(kf0, kf1, kf2, kf3, kf4, kf5, kf6, kf7, kf8, kf9, kf10, kf11, kf12, kf13, kf14, kf15, kf16, kf17, kf18, kf19, kf20, kf21);
        }
        return anim;
    }

    public void setData(VisualizationVariablesContainer data) {
        this.data = data;
        blockIndex = 0;
        phaseIndex = 0;
        refresh();
    }

    public void setBlockIndex(int index) {
        if (blockIndex != index && index >= 0) {
            blockIndex = index;
            phaseIndex = 0;
            refresh();
        }
    }

    private void refresh() {
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                cells[j][i].text.setText(String.format("%02X", data.getPhase(blockIndex, phaseIndex)[j][i]));
                if (phaseIndex == 40) {
                    cells[j][i].rectangle.setFill(Color.LIGHTSTEELBLUE);
                } else {
                    cells[j][i].rectangle.setFill(Color.RED);
                }
                cells[j][i].layoutXProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(i + 1));
                cells[j][i].layoutYProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(j));
            }
        }
        
        roundNameLabel.setText(phaseNames[phaseIndex]);
        animationText.getChildren().clear();
        phaseIndexSlider.setValue(phaseIndex);
        
        int animIndex = (phaseIndex - 1) % 4;
        
        if (phaseIndex == 0 || phaseIndex == 39) {
            middleText.setType(MiddleTextBlock.typeOfDescription.ADD_ROUND_KEY);
            if (phaseIndex == 0) {
                roundNumberStringProperty.set("0");
            } else {
                roundNumberStringProperty.set("10");
            }
        } else if (encryptionMode) {
            switch(animIndex) {
                case 0:
                    middleText.setType(MiddleTextBlock.typeOfDescription.SUBSTITUTE_BYTES);
                    break;
                case 1:
                    middleText.setType(MiddleTextBlock.typeOfDescription.SHIFT_ROWS);
                    break;
                case 2:
                    middleText.setType(MiddleTextBlock.typeOfDescription.MIX_COLUMNS);
                    break;
                case 3:
                    middleText.setType(MiddleTextBlock.typeOfDescription.ADD_ROUND_KEY);
                    break;
            }
            roundNumberStringProperty.set(String.valueOf((phaseIndex + 3) / 4));
        } else {
            switch(animIndex) {
                case 0:
                    middleText.setType(MiddleTextBlock.typeOfDescription.INVERSE_SHIFT_ROWS);
                    break;
                case 1:
                    middleText.setType(MiddleTextBlock.typeOfDescription.INVERSE_SUBTITUTE_BYTES);
                    break;
                case 2:
                    middleText.setType(MiddleTextBlock.typeOfDescription.ADD_ROUND_KEY);
                    break;
                case 3:
                    middleText.setType(MiddleTextBlock.typeOfDescription.INVERSE_MIXCOLUMNS);
                    break;
            }
            roundNumberStringProperty.set(String.valueOf((phaseIndex + 3) / 4));
        }
        
        lastRoundProperty.setValue(phaseIndex == 40);
        if (keyField != null) {
            if (encryptionMode) {
                keyField.refresh(Integer.parseInt(roundNumberStringProperty.get()));
            } else {
                keyField.refresh(10 - Integer.parseInt(roundNumberStringProperty.get()));
            }
        }
        
    }

    private void setConstraints() {
        ColumnConstraints cc0 = new ColumnConstraints();
        cc0.setMaxWidth(Double.MAX_VALUE);
        cc0.setHgrow(Priority.NEVER);
        cc0.setMinWidth(50);
        cc0.setMaxWidth(50);

        ColumnConstraints cc1 = new ColumnConstraints();
        cc1.setMaxWidth(Double.MAX_VALUE);
        cc1.setHgrow(Priority.ALWAYS);

        ColumnConstraints cc2 = new ColumnConstraints();
        cc2.setMaxWidth(Double.MAX_VALUE);
        cc2.setHgrow(Priority.NEVER);
        cc2.setMinWidth(50);
        cc2.setMaxWidth(50);

        RowConstraints rc0 = new RowConstraints();
        rc0.setVgrow(Priority.NEVER);

        RowConstraints rc1 = new RowConstraints();
        rc1.setVgrow(Priority.NEVER);
        rc1.minHeightProperty().bind(((Region) stateArrayCellsContainer.getChildren().get(0)).widthProperty());
        rc1.maxHeightProperty().bind(((Region) stateArrayCellsContainer.getChildren().get(0)).widthProperty());

        RowConstraints rc2 = new RowConstraints();
        rc2.setVgrow(Priority.ALWAYS);
        rc2.minHeightProperty().bind(((Region) stateArrayCellsContainer.getChildren().get(0)).widthProperty().multiply(4));
        rc2.maxHeightProperty().bind(((Region) stateArrayCellsContainer.getChildren().get(0)).widthProperty().multiply(4));
        RowConstraints rc3 = new RowConstraints();
        rc3.setVgrow(Priority.NEVER);
        RowConstraints rc4 = new RowConstraints();
        rc4.setVgrow(Priority.NEVER);

        getRowConstraints().addAll(rc0, rc1, rc2, rc3, rc4);
        getColumnConstraints().addAll(cc0, cc1, cc2);
    }

    private void creatingStackPanes() {
        phaseIndex = 0;
        cells = new StateArrayCell[4][4];
        stateArrayCellsContainer = new Pane();
        stateArrayCellsContainer.setMinSize(0, 0);
        stateArrayCellsContainer.setMaxSize(Double.MAX_VALUE, Double.MAX_VALUE);
        stateArrayCellsContainer.setMinSize(0, 0);
        GridPane.setHgrow(stateArrayCellsContainer, Priority.ALWAYS);
        GridPane.setVgrow(stateArrayCellsContainer, Priority.ALWAYS);
        GridPane.setHgrow(this, Priority.ALWAYS);
        GridPane.setVgrow(this, Priority.ALWAYS);
        
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                StateArrayCell cell = new StateArrayCell(Integer.toString(i) + Integer.toString(j));
                cells[j][i] = cell;
                cell.layoutXProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(i + 1));
                cell.layoutYProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(j));
                stateArrayCellsContainer.getChildren().add(cell);
            }
        }
        
        stateArrayCellsContainer.maxHeightProperty().bind(((Region) stateArrayCellsContainer.getChildren().get(0)).widthProperty().multiply(4));
        add(stateArrayCellsContainer, 1, 2);
    }

    private void Initialization() {
        setMinSize(0, 0);
        animation = new Timeline();
        this.widthProperty().addListener(new InvalidationListener() {

            @Override
            public void invalidated(Observable observable) {
                invalidatedAnimation();
            }
        });
        roundLabel = new Label("round: ");
        roundLabel.setFont(new Font(15));
        roundNumberStringProperty = new SimpleStringProperty("0");
        roundNumberLabel = new Label();
        roundNumberLabel.setFont(new Font(15));
        HBox roundNumberHbox = new HBox(3);
        roundNumberHbox.getChildren().addAll(roundLabel, roundNumberLabel);
        GridPane.setFillHeight(roundNumberHbox, false);
        roundNumberLabel.textProperty().bind(roundNumberStringProperty);
        sliderStartLabel = new Label("PT");
        sliderEndLabel = new Label("CT");
        Tooltip ts = new Tooltip("PlainText");
        ts.setFont(toolTipFont);
        sliderStartLabel.setTooltip(ts);
        Tooltip te = new Tooltip("CipherText");
        te.setFont(toolTipFont);
        sliderEndLabel.setTooltip(te);
        sliderEndLabel.setPadding(new Insets(20, 0, 0, 0));
        sliderStartLabel.setPadding(new Insets(20, 0, 0, 0));
        GridPane.setHalignment(sliderStartLabel, HPos.CENTER);
        GridPane.setHalignment(sliderEndLabel, HPos.CENTER);
        GridPane.setValignment(sliderStartLabel, VPos.CENTER);
        GridPane.setValignment(sliderEndLabel, VPos.CENTER);
        add(sliderStartLabel, 0, 4);
        add(sliderEndLabel, 2, 4);
        phaseIndexSlider = new Slider();
        phaseIndexSlider.setFocusTraversable(false);
        phaseIndexSlider.setPadding(new Insets(20, 0, 0, 0));
        GridPane.setValignment(phaseIndexSlider, VPos.CENTER);
        phaseIndexSlider.setBlockIncrement(1.0);
        phaseIndexSlider.setMin(0);
        phaseIndexSlider.setMax(40);
        phaseIndexSlider.valueProperty().addListener(new ChangeListener<Number>() {

            @Override
            public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number newValue) {
                phaseIndex = newValue.intValue();
                indexPropertyPreviousButton = new SimpleBooleanProperty(phaseIndex <= 0);
                indexPropertyNextButton = new SimpleBooleanProperty(phaseIndex >= 40);
                buttonNext.disableProperty().bind(Bindings.or(indexPropertyNextButton, animation.statusProperty()
                        .isEqualTo(Animation.Status.RUNNING)));
                buttonPrevious.disableProperty().bind(Bindings.or(indexPropertyPreviousButton, animation.statusProperty()
                        .isEqualTo(Animation.Status.RUNNING)));
                operationIndexLabel.setText(String.format(String.format("%02d / 40", (int) phaseIndexSlider.getValue())));
                refresh();
            }
        });
        add(phaseIndexSlider, 1, 4);
        buttonPrevious = new Button("<<");
        Tooltip tprev = new Tooltip("Previous operation");
        tprev.setFont(toolTipFont);
        buttonPrevious.setTooltip(tprev);
        buttonPrevious.setMinWidth(40);
        buttonPrevious.setMaxWidth(40);
        GridPane.setHgrow(buttonPrevious, Priority.NEVER);
        GridPane.setVgrow(buttonPrevious, Priority.NEVER);
        GridPane.setHalignment(buttonPrevious, HPos.RIGHT);
        indexPropertyPreviousButton = new SimpleBooleanProperty(phaseIndex <= 0);
        buttonPrevious.disableProperty().bind(Bindings.or(indexPropertyPreviousButton, animation.statusProperty()
                .isEqualTo(Animation.Status.RUNNING)));
        buttonPrevious.addEventHandler(MouseEvent.MOUSE_CLICKED, new javafx.event.EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                if (phaseIndex - 1 >= 0) {
                    roundNameLabel.setText(phaseNames[--phaseIndex]);
                    indexPropertyPreviousButton.set(phaseIndex <= 0);
                    indexPropertyNextButton.set(phaseIndex >= 40);
                    refresh();
                }
            }
        });
        buttonNext = new Button(">>");
        Tooltip tnext = new Tooltip("Next operation");
        tnext.setFont(toolTipFont);
        buttonNext.setTooltip(tnext);
        buttonNext.setMinWidth(40);
        buttonNext.setMaxWidth(40);
        GridPane.setHgrow(buttonNext, Priority.NEVER);
        GridPane.setVgrow(buttonNext, Priority.NEVER);
        GridPane.setHalignment(buttonNext, HPos.LEFT);
        indexPropertyNextButton = new SimpleBooleanProperty(phaseIndex >= 40);
        buttonNext.disableProperty().bind(Bindings.or(indexPropertyNextButton, animation.statusProperty()
                .isEqualTo(Animation.Status.RUNNING)));

        buttonNext.addEventHandler(MouseEvent.MOUSE_CLICKED, new javafx.event.EventHandler<MouseEvent>() {

            @Override
            public void handle(MouseEvent event) {
                if (phaseIndex + 1 <= phaseNames.length - 1) {
                    roundNameLabel.setText(phaseNames[++phaseIndex]);
                    indexPropertyNextButton.set(phaseIndex >= 40);
                    indexPropertyPreviousButton.set(phaseIndex <= 0);
                    refresh();
                }
            }
        });
        buttonStart = new Button("►");
        buttonStart.setMinWidth(35);
        buttonStart.setMaxWidth(35);
        lastRoundProperty = new SimpleBooleanProperty(phaseIndex == 40);
        buttonStart.disableProperty().bind(Bindings.or(animation.statusProperty().isEqualTo(Animation.Status.RUNNING), lastRoundProperty));
        Tooltip tsta = new Tooltip("Play the animation");
        tsta.setFont(toolTipFont);
        buttonStart.setTooltip(tsta);
        buttonPause = new Button("װ");
        buttonPause.setMinWidth(35);
        buttonPause.setMaxWidth(35);
        buttonPause.disableProperty().bind(animation.statusProperty().isNotEqualTo(Animation.Status.RUNNING));
        Tooltip tp = new Tooltip("Pause the animation");
        tp.setFont(toolTipFont);
        buttonPause.setTooltip(tp);
        buttonStop = new Button("■");
        buttonStop.setMinWidth(35);
        buttonStop.setMaxWidth(35);
        buttonStop.disableProperty().bind(animation.statusProperty().isNotEqualTo(Animation.Status.RUNNING));
        Tooltip tst = new Tooltip("Stop the animation");
        tst.setFont(toolTipFont);
        buttonStop.setTooltip(tst);
        phaseNames = new String[41];
        roundNameLabel = new Label();
        roundNameLabel.setTextFill(Color.ORCHID);
        roundNameLabel.setFont(new Font(18));
        switchToEncryption();
        GridPane.setHalignment(roundNameLabel, HPos.CENTER);
        add(roundNameLabel, 0, 0, 3, 1);
        add(buttonPrevious, 0, 2);
        add(buttonNext, 2, 2);
        HBox hbox = new HBox(5);
        hbox.setAlignment(Pos.CENTER);
        GridPane.setMargin(hbox, new Insets(10, 0, 0, 0));
        hbox.getChildren().addAll(buttonStart, buttonPause, buttonStop);
        add(hbox, 1, 3);
        add(roundNumberHbox, 0, 0, 2, 1);
        HBox textWrapper = new HBox();
        textWrapper.setFillHeight(false);
        textWrapper.setAlignment(Pos.CENTER);
        GridPane.setValignment(textWrapper, VPos.CENTER);
        animationText = new TextFlow();
        animationText.setTextAlignment(TextAlignment.CENTER);
        textWrapper.getChildren().add(animationText);
        operationIndexLabel = new Label();
        add(operationIndexLabel, 1, 0, 2, 1);
        operationIndexLabel.setAlignment(Pos.CENTER);
        GridPane.setHalignment(operationIndexLabel, HPos.RIGHT);
        operationIndexLabel.setText(String.format(String.format("%02d / 40", (int) phaseIndexSlider.getValue())));
        Tooltip t = new Tooltip("Current operation index");
        t.setFont(toolTipFont);
        operationIndexLabel.setTooltip(t);
        operationIndexLabel.setFont(new Font(15));
        add(textWrapper, 0, 1, 3, 1);
    }

    public void switchToDecryption() {
        animation.stop();
        if (encryptionMode) {
            keyField.refresh(Integer.parseInt(roundNumberStringProperty.get()));
        } else {
            keyField.refresh(10 - Integer.parseInt(roundNumberStringProperty.get()));
        }
        sbox.refresh();
        encryptionMode = false;
        phaseNames[0] = "Add round key";
        for (int i = 0; i < 36; i++) {
            switch(i % 4) {
                case 0:
                    phaseNames[i + 1] = ("Inverse shift rows");
                    break;
                case 1:
                    phaseNames[i + 1] = ("Inverse substitute bytes");
                    break;
                case 2:
                    phaseNames[i + 1] = ("Add round key");
                    break;
                case 3:
                    phaseNames[i + 1] = ("Inverse mix columns");
                    break;
            }
        }
        phaseNames[37] = "Inverse shift rows";
        phaseNames[38] = "Inverse substitute bytes";
        phaseNames[39] = "Add round key";
        phaseNames[39] = "Add round key";
        roundNameLabel.setText(phaseNames[0]);
        phaseIndex = 0;
        indexPropertyPreviousButton.set(phaseIndex <= 0);
        indexPropertyNextButton.set(phaseIndex >= 40);
        sliderStartLabel.setText("CT");
        sliderEndLabel.setText("PT");
        sliderEndLabel.getTooltip().setText("PlainText");
        sliderStartLabel.getTooltip().setText("CipherText");
    }

    public void switchToEncryption() {
        animation.stop();
        if (keyField != null && sbox != null) {
            sbox.refresh();
            if (encryptionMode) {
                keyField.refresh(Integer.parseInt(roundNumberStringProperty.get()));
            } else {
                keyField.refresh(10 - Integer.parseInt(roundNumberStringProperty.get()));
            }
        }
        encryptionMode = true;
        phaseNames[0] = "Add round key";
        for (int i = 0; i < 36; i++) {
            switch(i % 4) {
                case 0:
                    phaseNames[i + 1] = ("Substitute bytes");
                    break;
                case 1:
                    phaseNames[i + 1] = ("Shift rows");
                    break;
                case 2:
                    phaseNames[i + 1] = ("Mix columns");
                    break;
                case 3:
                    phaseNames[i + 1] = ("Add round key");
                    break;
            }
        }
        phaseNames[37] = "Substitute bytes";
        phaseNames[38] = "Shift rows";
        phaseNames[39] = "Add round key";
        phaseNames[40] = "Output";
        roundNameLabel.setText(phaseNames[0]);
        phaseIndex = 0;
        indexPropertyPreviousButton.set(phaseIndex <= 0);
        indexPropertyNextButton.set(phaseIndex >= 40);
        sliderStartLabel.setText("PT");
        sliderEndLabel.setText("CT");
        sliderEndLabel.getTooltip().setText("CipherText");
        sliderStartLabel.getTooltip().setText("PlainText");
    }

    private class StateArrayCell extends StackPane {

        @FXML
        Text text;

        @FXML
        Rectangle rectangle;

        public StateArrayCell(String text) {
            this.setMinSize(0, 0);
            this.text = new Text(text);
            this.text.setFont(Font.font("system", FontWeight.BOLD, 15));
            rectangle = new Rectangle();
            rectangle.widthProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).subtract(3));
            rectangle.heightProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).subtract(3));
            this.maxHeightProperty().bind(stateArrayCellsContainer.widthProperty().divide(6));
            this.maxWidthProperty().bind(stateArrayCellsContainer.widthProperty().divide(6));
            this.minHeightProperty().bind(stateArrayCellsContainer.widthProperty().divide(6));
            this.minWidthProperty().bind(stateArrayCellsContainer.widthProperty().divide(6));
            rectangle.setArcHeight(7);
            rectangle.setArcWidth(7);
            rectangle.setFill(Color.RED);
            rectangle.setStroke(Color.BLACK);
            this.getChildren().addAll(rectangle, this.text);
        }

        public void setText(String text) {
            this.text.setText(text);
        }

        public String getText() {
            return this.text.getText();
        }
    }

    private void invalidatedAnimation() {
        layout();
        int animIndex = (phaseIndex - 1) % 4;
        if (((animation.getStatus().equals(Timeline.Status.RUNNING) || animation.getStatus().equals(Timeline.Status.PAUSED)) && encryptionMode && animIndex == 1) || ((animation.getStatus().equals(Timeline.Status.RUNNING) || animation.getStatus().equals(Timeline.Status.PAUSED)) && !encryptionMode && animIndex == 0)) {
            Animation.Status status = animation.getStatus();
            Duration currentTime = animation.getCurrentTime();
            animation.stop();
            for (int i = 0; i < 4; i++) {
                for (int j = 0; j < 4; j++) {
                    cells[j][i].layoutXProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(i + 1));
                    cells[j][i].layoutYProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(j));
                    cells[j][i].rectangle.setFill(Color.RED);
                }
            }
            if (encryptionMode) {
                animation = swiftRowsAnim();
            } else {
                animation = invereSwiftRowsAnim();
            }
            animation.playFrom(currentTime);
            if (status.equals(Timeline.Status.PAUSED)) {
                animation.pause();
            }
            refreshAnimationButtonProperties();
        } else if ((animation.getStatus().equals(Timeline.Status.STOPPED) && encryptionMode && animIndex == 1) || (animation.getStatus().equals(Timeline.Status.STOPPED) && !encryptionMode && animIndex == 0)) {
            for (int i = 0; i < 4; i++) {
                for (int j = 0; j < 4; j++) {
                    cells[j][i].layoutXProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(i + 1));
                    cells[j][i].layoutYProperty().bind(stateArrayCellsContainer.widthProperty().divide(6).multiply(j));
                }
            }
        }
    }
}
