package javafxmainapplication;

import aes.AESWithAllPhasesStored;
import aes.VisualizationVariablesContainer;
import customControls.KeyFieldsControl;
import customControls.MiddleTextBlock;
import customControls.SBoxInverseSBoxControl;
import customControls.PlainTextCipherTextControl;
import customControls.StateArrayControl;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.ResourceBundle;
import javafx.beans.InvalidationListener;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.Region;
import javafx.stage.Stage;

public class FXML_MainWindowController implements Initializable {

    @FXML
    private PlainTextCipherTextControl inputOutput;

    @FXML
    private StateArrayControl stateArray;

    @FXML
    private SBoxInverseSBoxControl sbox;

    @FXML
    private KeyFieldsControl keyField;

    @FXML
    private Label tittleLabel;

    @FXML
    private Button buttonSwitch;

    @FXML
    private Label sBoxLabel;

    @FXML
    private MiddleTextBlock middleText;

    private byte[] key;

    private boolean encryption;

    @Override
    public void initialize(URL url, ResourceBundle rb) {
        encryption = true;
    }

    public void setPlainText(byte[] input) {
        inputOutput.setInput(input);
    }

    public void setCipherText(byte[] input) {
        inputOutput.setOutput(input);
    }

    public void fillPlainText(int offset) {
        inputOutput.fillPlainText(offset);
    }

    public void set(VisualizationVariablesContainer data) {
        stateArray.setData(data);
    }

    public void setStataArrayToInputOutputControl() {
        inputOutput.setStateArray(stateArray);
    }

    public void setSboxToStateArray() {
        stateArray.setSBox(sbox);
    }

    public void setKey(byte[] key) {
        keyField.setValues(key);
    }

    public void setOriginalKey(byte[] key) {
        this.key = key;
    }

    public void setKeyFieldToStateArray() {
        stateArray.setkeyField(keyField);
    }

    public void setMiddleTextToStateArray() {
        stateArray.setMiddleText(middleText);
    }

    public void setPlainTextCipherTextControlToStateArray() {
        stateArray.setPlainTextCipherTextControl(inputOutput);
    }

    @FXML
    private void handleSwitchingBetweenEncrDecr(ActionEvent event) throws Exception {
        AESWithAllPhasesStored aes = new AESWithAllPhasesStored();
        if (encryption) {
            encryption = false;
            tittleLabel.setText("Decryption Visualization");
            sbox.setToInverseSBox();
            buttonSwitch.setText("show encryption");
            stateArray.switchToDecryption();
            set(aes.decrypt(inputOutput.getOutput(), key));
            inputOutput.swithToDecrEncr();
            sBoxLabel.setText("Inverse SBox");
        } else {
            encryption = true;
            tittleLabel.setText("Encryption Visualization");
            sbox.setToSBox();
            buttonSwitch.setText("show decryption");
            stateArray.switchToEncryption();
            set(aes.encrypt(new String(inputOutput.getOutput(), StandardCharsets.UTF_8), key));
            inputOutput.swithToDecrEncr();
            sBoxLabel.setText("SBox");
        }
    }

    @FXML
    private void handleExitingToInputScreen(ActionEvent event) throws Exception {
        Button b = (Button) event.getSource();
        Stage stageOld = (Stage) b.getScene().getWindow();
        Stage stage = new Stage();
        stage.setTitle("AES Visualization");
        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("FXML_StartPage.fxml"));
        Button btn = (Button) event.getSource();
        Parent root1 = (Parent) fxmlLoader.load();
        ((Region) root1).widthProperty().addListener(new InvalidationListener() {

            @Override
            public void invalidated(javafx.beans.Observable observable) {
                root1.layout();
            }
        });
        ((Region) root1).heightProperty().addListener(new InvalidationListener() {

            @Override
            public void invalidated(javafx.beans.Observable observable) {
                root1.layout();
            }
        });
        stage.setResizable(false);
        stage.setScene(new Scene(root1));
        stage.show();
        stage.centerOnScreen();
        stageOld.close();
    }
}
